#!/bin/bash

#
# Environment setup, modify it if needed
#
DATE=`date '+%y%m%d_%H%M'`
PWD=`pwd`
TFTPROOT=/tftpboot

. ./USERCONFIG
# add for custom upgrade verify
. ./CustomFlag.conf

#
# Toolchains Environment
#
export TOOLCHAIN_TOP=/opt/toolchain_mips_nofpu
export TOOLCHAIN_PATH=${TOOLCHAIN_TOP}
export CROSS_COMPILE_PATH=${TOOLCHAIN_PATH}/bin
export CROSS_COMPILE=${TOOLCHAIN_PATH}/bin/mips-linux-
export STRIP=${CROSS_COMPILE}strip
export OBJCOPY=${CROSS_COMPILE}objcopy
export RANLIB=${CROSS_COMPILE}ranlib

#
# Project Environment
#
PROJECT_TOP=${PWD}
TOOLS_DIR=${PROJECT_TOP}/host/tools
IMAGE_DIR=${PROJECT_TOP}/image

OPENSOURE_DIR=${PROJECT_TOP}/core/userapps/opensource
PRIVATE_DIR=${PROJECT_TOP}/core/userapps/private
INCLUDE_DIR=${PROJECT_TOP}/core/userapps/include
APPS_DIR=${PROJECT_TOP}/target/apps
ROOTFS_DIR=${PROJECT_TOP}/target/rootfs

VENDOR_DIR=${PROJECT_TOP}/vendor



export MACHINE=mips
export ARCH=mips
export TARGET_PLATFORM=mips-linux-uclibc

export INSTALL_DIR=${ROOTFS_DIR}/rootfs

export PATH=${CROSS_COMPILE_PATH}:$PATH

BUSYBOX_DIR=${OPENSOURE_DIR}/busybox
DNSMASQ_DIR=${OPENSOURE_DIR}/dnsmasq
EBTABLES_DIR=${OPENSOURE_DIR}/ebtables
IGMPPROXY_DIR=${OPENSOURE_DIR}/igmpproxy
IPTABLES_DIR=${OPENSOURE_DIR}/iptables
L2TP_DIR=${OPENSOURE_DIR}/l2tp
LIBUPNP_DIR=${OPENSOURE_DIR}/libupnp
UPNPD_DIR=${OPENSOURE_DIR}/linuxigd
MTDUTIL_DIR=${OPENSOURE_DIR}/mtd-util
NTPCLIENT_DIR=${OPENSOURE_DIR}/ntpclient
PPPD_DIR=${OPENSOURE_DIR}/ppp
PPTP_DIR=${OPENSOURE_DIR}/pptp
PPPOE_DIR=${OPENSOURE_DIR}/rp-pppoe
SSMTP_DIR=${OPENSOURE_DIR}/ssmtp
UDHCP_DIR=${OPENSOURE_DIR}/udhcp
UPDATEDD_DIR=${OPENSOURE_DIR}/updatedd

AUTOVER_DIR=${PRIVATE_DIR}/autover
DATASTORE_CLIENT_DIR=${PRIVATE_DIR}/datastore_client
GEN_DNS=${PRIVATE_DIR}/gen_dns
IFINIT_DIR=${PRIVATE_DIR}/ifInit
TR069_DIR=${PRIVATE_DIR}/OpenAgent
SHARELIB_DIR=${PRIVATE_DIR}/sharelib
UDAEMON_DIR=${PRIVATE_DIR}/udaemon
HTTPD_DIR=${PRIVATE_DIR}/newuser_if/httpd
NAS_DIR=${PRIVATE_DIR}/newuser_if/NAS
SERVICE_DIR=${PRIVATE_DIR}/newuser_if/service

#
# Turnkey Environment
#
TURNKEY=rtl867x
export TURNKEY_TOP=${VENDOR_DIR}/${TURNKEY}/uClinux-dist
TURNKEY_DRV_DIR=${TURNKEY_TOP}
TURNKEY_DRV_OPENSOURCE_DIR=${TURNKEY_TOP}
TURNKEY_BOOT_ROM_DIR=${TURNKEY_TOP}/boot
export TURNKEY_KERNEL_DIR=${TURNKEY_TOP}/linux-2.6.x
TURNKEY_TARGETS_DIR=${TURNKEY_TOP}/romfs
export TURNKEY_USERSPACE_DIR=${TURNKEY_TOP}/user

export TURNKEY_HOSTTOOLS_DIR=${TURNKEY_TOP}/hostTools
export TURNKEY_GPL_DIR=${TURNKEY_USERSPACE_DIR}/gpl
export TURNKEY_PRIVATE_DIR=${TURNKEY_USERSPACE_DIR}/private
export TURNKEY_PUBLIC_DIR=${TURNKEY_USERSPACE_DIR}/public

export LEN_CHECKSUM_CAL_ENV=4
# tips: we can give the value from command line, SO set it to no if not require it
# e.g: DO_TWO_TIME_KER=no ./build_project.sh turnkey
if [ -z "$DO_TWO_TIME_KER" ]; then
	export DO_TWO_TIME_KER="yes"
fi

TURNKEY_SRC_PACK=SDK2.6-20080917.tar.bz2
TURNKEY_SRC_URL=http://172.22.102.213/svn/Realtek_867x_sdk/trunk/
TURNKEY_BOOT_URL=ftp://172.22.102.201//Project/SkyHopper/Vendor/BinImage/boot.bin

LIB_VER=0.9.26
export TARGET_LIB_PATH=${TURNKEY_TOP}/lib/uClibc

#
# Build Image Environment
#
MODEL_NAME=SkyHopper
PROFILE_NAME=${MODEL_NAME}

# Include version.make and profile Environment
#source ${TURNKEY_TOP}/.config
#source ${TURNKEY_TARGETS_DIR}/${MODEL_NAME}/${PROFILE_NAME}

BOOT_FILE=${TURNKEY_BOOT_ROM_DIR}/
FS_KERNEL_IMAGE_NAME=${MODEL_NAME}_fs_kernel
BOOT_FS_KERNEL_IMAGE_NAME=${MODEL_NAME}_boot_fs_kernel
FLASH_IMAGE_NAME=${MODEL_NAME}_flash_image

#
# Function definition
#
# function to build bootloader
function build_bootloader() {
    echo "Build Turnkey bootcode"
    cd ${TURNKEY_TOP}
    echo "No bootcode source availabe yet"
    #./build_turnkey.sh bootcode

    cd ${PROJECT_TOP}
    return 0
}

# function to clean bootloader
function clean_bootloader() {
    echo "Clean Turnkey bootcode"
    cd ${TURNKEY_TOP}
    #./build_turnkey.sh bootcode-clean

    cd ${PROJECT_TOP}
    return 0
}

# function to build turnkey
function build_fs_kernel() {
    echo "Build Turnkey(fs, kernel and image)"
    cd ${TURNKEY_TOP}
    
    make
    
    if [ $? -ne 0 ];then
	exit 0
    fi

    cd ${PROJECT_TOP}
    return 0
}

# function to build turnkey
function build_fs() {
    echo "Build Turnkey User application)"
    cd ${TURNKEY_TOP}
    
    make user_only
    
    if [ $? -ne 0 ];then
	exit 0
    fi
    
    make user_romfs
    if [ $? -ne 0 ];then
	exit 0
    fi    
    
    #make image

    cd ${PROJECT_TOP}
    return 0
}

# function to clean kernel
function clean_fs_kernel() {
    echo "Clean Turnkey kernel"
    cd ${TURNKEY_TOP}
    make clean

    cd ${PROJECT_TOP}
    return 0
}

#function to clean romfs, rootfs, chmod  777

function clean_fs_romfs(){
	echo "Clean Turnkey (romfs ,rootfs, chmod)"
	echo "chmod 777 the whole turnkey, please wait ...."
	sudo chmod -R 777 ./*
	echo "clean romfs & rootfs"
	cd ${TURNKEY_TOP}
	if [ -d romfs ];then
	   rm -rf romfs
	fi
	
	if [ -r rootfs ]; then
           rm -rf rootfs
	fi
	echo "clean romfs & rootfs done"
}
# function to clean turnkey
function clean_turnkey_boa_LINUX() {
    echo "Clean boa & cwmpClient"
    
    cd ${TURNKEY_USERSPACE_DIR}/boa/src/LINUX/
    rm -rf *.o *.a *.so
    
    cd ${TURNKEY_USERSPACE_DIR}/cwmp-tr069/cwmpClient/
    rm -rf *.o *.a *.so cwmpClient

    cd ${PROJECT_TOP}
    return 0
}

# function to build rootfs
function build_rootfs() {
    echo "Check rootfs"
    if [ ! -d ${ROOTFS_DIR}/rootfs ]; then
        echo "Build ${ROOTFS_DIR}/rootfs"
        cd ${ROOTFS_DIR}
        cp -R init_rootfs rootfs
        find ${ROOTFS_DIR}/rootfs -name .svn | xargs rm -rf
        #tar zxvf rootfs.tgz
        cd ${PROJECT_TOP}
    fi

    cd ${PROJECT_TOP}
    return 0
}

# function to clean rootfs
function clean_rootfs() {
    echo "Clean ${ROOTFS_DIR}/rootfs"
    cd ${ROOTFS_DIR}
    rm -rf rootfs

    cd ${PROJECT_TOP}
    return 0
}

# function to build busybox
function build_busybox() {
    echo "Clean original busybox utility and symbolic link"
    if [ -d ${ROOTFS_DIR}/rootfs/bin ]; then
        rm -rf ${ROOTFS_DIR}/rootfs/bin
    fi
    if [ -d ${ROOTFS_DIR}/rootfs/sbin ]; then
        rm -rf ${ROOTFS_DIR}/rootfs/sbin
    fi
    if [ -d ${ROOTFS_DIR}/rootfs/usr/bin ]; then
        rm -rf ${ROOTFS_DIR}/rootfs/usr/bin
    fi
    if [ -d ${ROOTFS_DIR}/rootfs/usr/sbin ]; then
        rm -rf ${ROOTFS_DIR}/rootfs/usr/sbin
    fi

    echo "Build busybox utility"
    make -w -C ${BUSYBOX_DIR} ARCH=${ARCH} CROSS_COMPILE=${CROSS_COMPILE} V=1
    echo "Install busybox utility"
    make -w -C ${BUSYBOX_DIR} ARCH=${ARCH} CROSS_COMPILE=${CROSS_COMPILE} CONFIG_PREFIX=${ROOTFS_DIR}/rootfs install V=1

    cd ${PROJECT_TOP}
    return 0
}

# function to install library
function install_library() {
    echo "Install library"
    cd ${ROOTFS_DIR}/rootfs/lib/
    rm -f *

    echo "Install toolchain library"
    cp -f ${TARGET_LIB_PATH}/ld-uClibc-$(LIB_VER).so .
    cp -f ${TARGET_LIB_PATH}/libuClibc-$(LIB_VER).so .
    cp -f ${TARGET_LIB_PATH}/libnsl-$(LIB_VER).so .
    cp -f ${TARGET_LIB_PATH}/libcrypt-$(LIB_VER).so .
    cp -f ${TARGET_LIB_PATH}/libm-$(LIB_VER).so .
    cp -f ${TARGET_LIB_PATH}/libdl-$(LIB_VER).so .
    cp -f ${TARGET_LIB_PATH}/libresolv-$(LIB_VER).so .
    cp -f ${TARGET_LIB_PATH}/libutil-$(LIB_VER).so .
    cp -f ${TARGET_LIB_PATH}/libpthread-$(LIB_VER).so .
    cp -f ${TOOLCHAIN_PATH}/mips-linux-uclibc/lib/libgcc_s.so.1 .
    ln -s ld-uClibc-$(LIB_VER).so ld-uClibc.so.0
    ln -s libuClibc-$(LIB_VER).so libc.so.0
    ln -s libnsl-$(LIB_VER).so libnsl.so.0
    ln -s libcrypt-$(LIB_VER).so libcrypt.so.0
    ln -s libm-$(LIB_VER).so libm.so.0
    ln -s libdl-$(LIB_VER).so libdl.so.0
    ln -s libresolv-$(LIB_VER).so libresolv.so.0
    ln -s libutil-$(LIB_VER).so libutil.so.0
    ln -s libpthread-$(LIB_VER).so libpthread.so.0
    ${STRIP} ld-uClibc-$(LIB_VER).so
    ${STRIP} libuClibc-$(LIB_VER).so
    ${STRIP} libnsl-$(LIB_VER).so
    ${STRIP} libcrypt-$(LIB_VER).so
    ${STRIP} libm-$(LIB_VER).so
    ${STRIP} libdl-$(LIB_VER).so
    ${STRIP} libresolv-$(LIB_VER).so
    ${STRIP} libutil-$(LIB_VER).so
    ${STRIP} libpthread-$(LIB_VER).so
    ${STRIP} libgcc_s.so.1

    cd ${PROJECT_TOP}
    return 0
}

# function to build share library
function build_share_library() {
    echo "Build share library"
    make -w -C ${SHARELIB_DIR} CROSS=${CROSS_COMPILE} all
    make -w -C ${SHARELIB_DIR}/oidlib CROSS=${CROSS_COMPILE} all

    cd ${PROJECT_TOP}
    return 0
}

# function to install share library
function install_share_library() {
    echo "Install share library"
    cp -f ${SHARELIB_DIR}/libshared.so ${ROOTFS_DIR}/rootfs/lib
    cp -f ${SHARELIB_DIR}/oidlib/liboid.a ${ROOTFS_DIR}/rootfs/lib

    cd ${PROJECT_TOP}
    return 0
}

# function to install vendor modules
function install_vendor_modules() {
    echo "Check rootfs"
    if [ ! -d ${ROOTFS_DIR}/rootfs ]; then
        echo "ERROR : Install modules fail! Please build rootfs first!"
        return 1
    fi

    echo "Install Turnkey Library"
    cd ${ROOTFS_DIR}/rootfs/lib

    echo "Install Turnkey Modules"
    cd ${ROOTFS_DIR}/rootfs/lib
    rm -rf modules
    cp -af $(TURNKEY_TARGETS_DIR)/lib/modules .

    if [ ! -d ${ROOTFS_DIR}/rootfs/etc/adsl ]; then
        mkdir ${ROOTFS_DIR}/rootfs/etc/adsl
    fi
    cp -f ${TURNKEY_DRV_Turnkey_DIR}/char/adsl/bcm96368/adsl_phy.bin ${ROOTFS_DIR}/rootfs/etc/adsl

    cd ${PROJECT_TOP}
    return 0
}

# function to install vendor applications
function install_vendor_apps() {
    echo "Install Turnkey Applications"
    echo "Check rootfs"
    if [ ! -d ${ROOTFS_DIR}/rootfs ]; then
        echo "ERROR : Install applications fail! Please build rootfs first!"
        return 1
    fi

    echo "Install bridge utility"
    cp -af $(TURNKEY_TARGETS_DIR)/bin/brctl ${ROOTFS_DIR}/rootfs/sbin

    echo "Install ethctl"
    cp -af $(TURNKEY_TARGETS_DIR)/bin/ethctl ${ROOTFS_DIR}/rootfs/sbin

    echo "Install xdslctl"
    cp -af $(TURNKEY_TARGETS_DIR)/bin/xdslctl ${ROOTFS_DIR}/rootfs/sbin
    cp -af $(TURNKEY_TARGETS_DIR)/bin/adsl ${ROOTFS_DIR}/rootfs/sbin
    cp -af $(TURNKEY_TARGETS_DIR)/bin/adslctl ${ROOTFS_DIR}/rootfs/sbin

    echo "Install xtmctl"
    cp -af $(TURNKEY_TARGETS_DIR)/bin/xtmctl ${ROOTFS_DIR}/rootfs/sbin
    cp -af $(TURNKEY_TARGETS_DIR)/bin/xtm ${ROOTFS_DIR}/rootfs/sbin

    echo "Install cmf"
    cp -af $(TURNKEY_TARGETS_DIR)/bin/cmfctl ${ROOTFS_DIR}/rootfs/sbin

    echo "Install ippd"
    cp -af $(TURNKEY_TARGETS_DIR)/bin/ippd ${ROOTFS_DIR}/rootfs/sbin

#    echo "Install wireless"

#    echo "Install wireless library"

#    echo "Install voice"
#    cp -af $(TURNKEY_TARGETS_DIR)/bin/vodsl ${ROOTFS_DIR}/rootfs/sbin

    cd ${PROJECT_TOP}
    return 0
}

# function to build iptables
function build_iptables() {
    echo "Build iptables"
    make -w -C ${IPTABLES_DIR} KERNEL_DIR=${TURNKEY_KERNEL_DIR} CC=${CROSS_COMPILE}gcc LD=${CROSS_COMPILE}ld AR=${CROSS_COMPILE}ar V=1

    cd ${PROJECT_TOP}
    return 0
}

# function to install iptables
function install_iptables() {
    echo "Install iptables"
    cp -f ${IPTABLES_DIR}/iptables ${ROOTFS_DIR}/rootfs/sbin
    cp -f ${APPS_DIR}/iptables.rule ${ROOTFS_DIR}/rootfs/apps

    cd ${PROJECT_TOP}
    return 0
}

# function to build auto_version
function build_autover() {
    echo "Build auto_version"
    cd ${AUTOVER_DIR}
    ./build

    cd ${PROJECT_TOP}
    return 0
}

# function to install auto_version
function install_autover() {
    echo "Install auto_version"
    cp -f ${AUTOVER_DIR}/autover ${TOOLS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build TR-069 agent
function build_TR_069_AGENT() {
    echo "Build TR-069 agent"
    make -w -C ${TR069_DIR} CROSS=${CROSS_COMPILE} OS_ROOT=${TURNKEY_KERNEL_DIR} VIGOR_ACS=1
    make -w -C ${TR069_DIR}/conf/Dllso_example clean
    make -w -C ${TR069_DIR}/conf/Dllso_example CROSS=${CROSS_COMPILE}

    cd ${PROJECT_TOP}
    return 0
}

# function to install TR-069 agent
function install_TR_069_AGENT() {
    echo "Install TR-069 agent"
    cp -f ${TR069_DIR}/src/agent ${APPS_DIR}
    cp -f ${TR069_DIR}/conf/Dllso_example/libdev.so ${ROOTFS_DIR}/rootfs/lib
    cp -f ${TR069_DIR}/conf/agent.conf ${APPS_DIR}
    cp -f ${TR069_DIR}/conf/param.xml ${APPS_DIR}
    cp -f ${TR069_DIR}/conf/param.xsd ${APPS_DIR}
    cp -f ${TR069_DIR}/conf/stun.conf ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build web server
function build_web_server() {
    echo "Build web server"
    cd ${NAS_DIR}
    ${TOOLS_DIR}/autover ${INCLUDE_DIR}/version.h

    # build Datastore and BOA
#    ./CleanAll.sh
    ./Build.sh

    cd ${PROJECT_TOP}
    return 0
}

# function to install web server
function install_web_server() {
    echo "Install web server"
    if [ ! -d $APPS_DIR/web ]; then
        mkdir $APPS_DIR/web
    fi
    cd  ${APPS_DIR}/web
    rm -rf *
    cp -rf ${SERVICE_DIR}/* .

    #install gen_dns
    if [ -e ${APPS_DIR}/gen_dns ]; then
        rm -rf ${APPS_DIR}/gen_dns
    fi
    cp -rf ${GEN_DNS} ${APPS_DIR}/.

    find ${APPS_DIR}/gen_dns -name .svn | xargs rm -rf

    # copy http to rootfs
    if [ ! -d ${APPS_DIR}/web/httpd ]; then
        mkdir ${APPS_DIR}/web/httpd
    fi
    cp -rf ${HTTPD_DIR}/* ${APPS_DIR}/web/httpd

    find ${APPS_DIR}/web -name .svn | xargs rm -rf
    ln -s /tmp ${APPS_DIR}/web/httpd/html/tmp

    # create datastoreDB folder
    if [ ! -d ${APPS_DIR}/web/datastoreDB ]; then
        mkdir ${APPS_DIR}/web/datastoreDB
    fi
    cp -f ${NAS_DIR}/datastore/output/datastore.dat ${APPS_DIR}/web/datastoreDB

    cd ${PROJECT_TOP}
    return 0
}

# function to build PPPoE server
function build_pppoe_server() {
    echo "Build PPPoE"
    make -w -C ${PPPOE_DIR}/src CROSS=${CROSS_COMPILE}

    echo "Build PPPD"
    make -w -C ${PPPD_DIR}/pppd MULTIPLE_PPPD=1 CROSS=${CROSS_COMPILE}
    make -w -C ${PPPD_DIR}/pppd/plugins CROSS=${CROSS_COMPILE}

    cd ${PROJECT_TOP}
    return 0
}

# function to install PPPoE server
function install_pppoe_server() {
    echo "Install PPPoE"
    cp -f ${PPPOE_DIR}/src/pppoe ${APPS_DIR}
    cp -f ${PPPOE_DIR}/scripts/pppoe-start ${APPS_DIR}
    cp -f ${PPPOE_DIR}/scripts/pppoe-stop ${APPS_DIR}
    cp -f ${PPPOE_DIR}/scripts/pppoe-connect ${APPS_DIR}
    cp -f ${PPPOE_DIR}/scripts/pppoe-status ${APPS_DIR}
    cp -f ${PPPOE_DIR}/scripts/pppoe_gen_cfg.sh ${APPS_DIR}

    echo "Install PPPD"
    cp -f ${PPPD_DIR}/pppd/pppd ${APPS_DIR}
    cp -f ${PPPD_DIR}/scripts/ip-up ${APPS_DIR}
    cp -f ${PPPD_DIR}/scripts/ip-pre-up ${APPS_DIR}
    cp -f ${PPPD_DIR}/scripts/ip-down ${APPS_DIR}
    cp -f ${PPPD_DIR}/scripts/pppoa-connect ${APPS_DIR}
    cp -f ${PPPD_DIR}/pppd/plugins/pppoatm/pppoatm.so ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build PPTP server
function build_pptp_server() {
    echo "Build PPTP"
    make -w -C ${PPTP_DIR} CROSS=${CROSS_COMPILE}

    cd ${PROJECT_TOP}
    return 0
}

# function to install PPTP server
function install_pptp_server() {
    echo "Install PPTP"
    cp -f ${PPTP_DIR}/pptp ${APPS_DIR}
    cp -f ${PPTP_DIR}/scripts/pptp-stop ${APPS_DIR}
    cp -f ${PPTP_DIR}/scripts/pptp-connect ${APPS_DIR}
    cp -f ${PPTP_DIR}/scripts/pptp_gen_cfg.sh ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build L2TP server
function build_l2tp() {
    echo "Build L2TP"
    make -w -C ${L2TP_DIR} CROSS=${CROSS_COMPILE}

    cd ${PROJECT_TOP}
    return 0
}

# function to install L2TP server
function install_l2tp() {
    echo "Install L2TP"
    cp -f ${L2TP_DIR}/l2tpd ${APPS_DIR}
    cp -f ${L2TP_DIR}/scripts/l2tpd_gen_cfg.sh ${APPS_DIR}
    cp -f ${L2TP_DIR}/scripts/l2tpd-stop ${APPS_DIR}
#    cp -f ${L2TP_DIR}/scripts/l2tpd-link ${APPS_DIR}
    cp -f ${L2TP_DIR}/scripts/l2tpd-connect ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build ifInit utility
function build_ifInit() {
    echo "Build ifInit"
    make -w -C ${IFINIT_DIR} CROSS=${CROSS_COMPILE} V=1

    cd ${PROJECT_TOP}
    return 0
}

# function to install ifInit utility
function install_ifInit() {
    echo "Install ifInit"
    cp -f ${IFINIT_DIR}/ifInit ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build DDNS utility
function build_DDNS_utility() {
    echo "Build DDNS"
    cd ${UPDATEDD_DIR}
    if [ -f ${UPDATEDD_DIR}/src/updatedd ]; then
        make clean
    fi
    ./configure --build=i386-linux --host=${TARGET_PLATFORM} --libdir=/lib
    make

    cd ${PROJECT_TOP}
    return 0
}

# function to install DDNS utility
function install_DDNS_utility() {
    echo "Install DDNS"
    if [ ! -d ${ROOTFS_DIR}/rootfs/lib/updatedd ]; then
        mkdir ${ROOTFS_DIR}/rootfs/lib/updatedd
    fi
    cp -f ${UPDATEDD_DIR}/src/updatedd ${APPS_DIR}
    cp -f ${UPDATEDD_DIR}/src/plugins/dyndns.so ${ROOTFS_DIR}/rootfs/lib/updatedd

    cd ${PROJECT_TOP}
    return 0
}

# function to build SSMTP utility
function build_SSMTP_utility() {
    echo "Build SSMTP"
    cd ${SSMTP_DIR}
    if [ -f ${SSMTP_DIR}/ssmtp ]; then
        make clean
    fi
    ./runme.sh

    cd ${PROJECT_TOP}
    return 0
}

# function to install SSMTP utility
function install_SSMTP_utility() {
    echo "Install SSMTP"
    cp -f ${SSMTP_DIR}/ssmtp ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build NTPCLIENT utility
function build_NTPCLIENT_utility() {
    echo "Build NTPCLIENT"
    cd ${NTPCLIENT_DIR}
    if [ -f ${NTPCLIENT_DIR}/ntpclient ]; then
        make clean
    fi
    ./runme.sh

    cd ${PROJECT_TOP}
    return 0
}

# function to install NTPCLEINT utility
function install_NTPCLIENT_utility() {
    echo "Install NTPCLEINT"
    cp -f ${NTPCLIENT_DIR}/ntpclient ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build UPnP utility
function build_UPNP_utility() {
    echo "Build UPnP"
    make -w -C ${LIBUPNP_DIR}/upnp TARGET=${TARGET_PLATFORM} CLIENT=0
    make -w -C ${UPNPD_DIR} CROSS=${CROSS_COMPILE} HAVE_LIBIPTC=1 LIBIPTC_PATH=${IPTABLES_DIR} LIBUPNP_PATH=${LIBUPNP_DIR}/upnp/bin/${TARGET_PLATFORM} UPNPINC_PATH=${LIBUPNP_DIR}/upnp

    cd ${PROJECT_TOP}
    return 0
}

# function to install UPnP utility
function install_UPNP_utility() {
    echo "Install UPnP"
    make -w -C ${LIBUPNP_DIR} install
    make -w -C ${UPNPD_DIR} TARGET_DIR=${ROOTFS_DIR}/rootfs install
    cp -f ${UPNPD_DIR}/upnpd ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build mtd-utility
function build_MTD_utility() {
    echo "Build MTD utility"
    make -w -C ${MTDUTIL_DIR}/zlib CROSS=${CROSS_COMPILE}
    make -w -C ${MTDUTIL_DIR} CROSS=${CROSS_COMPILE} OS_DIR=${TURNKEY_KERNEL_DIR} all

    cd ${PROJECT_TOP}
    return 0
}

# function to install mtd-utility
function install_MTD_utility() {
    echo "Install MTD utility"
    cp -f ${MTDUTIL_DIR}/flash_eraseall ${APPS_DIR}
    cp -f ${MTDUTIL_DIR}/flash_info ${APPS_DIR}
    cp -f ${MTDUTIL_DIR}/flashcp ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build dnsmasq
function build_dnsmasq_utility() {
    echo "Build dnsmasq"
    make -w -C ${DNSMASQ_DIR} CROSS=${CROSS_COMPILE}

    cd ${PROJECT_TOP}
    return 0
}

# function to install dnsmasq
function install_dnsmasq_utility() {
    echo "Install dnsmasq"
    cp -f ${DNSMASQ_DIR}/src/dnsmasq ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build igmpproxy
function build_igmpproxy() {
    echo "Build IGMP proxy"
    make -w -C ${IGMPPROXY_DIR}/src CROSS=${CROSS_COMPILE}

    cd ${PROJECT_TOP}
    return 0
}

# function to install igmpproxy
function install_igmpproxy() {
    echo "Install IGMP proxy"
    cp -f ${IGMPPROXY_DIR}/src/igmpproxy ${APPS_DIR}
    cp -f ${IGMPPROXY_DIR}/src/igmpproxy.conf ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to install igmpproxy
function install_igmpproxy_v3() {
    echo "Install IGMP proxy V3"
    cp -f $(TURNKEY_TARGETS_DIR)/bin/igmp ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build udaemon utility
function build_udaemon() {
    echo "Build udaemon"
    make -w -C ${UDAEMON_DIR} CROSS=${CROSS_COMPILE}

    cd ${PROJECT_TOP}
    return 0
}

# function to install udaemon utility
function install_udaemon() {
    echo "Install udaemon"
    cp -f ${UDAEMON_DIR}/udaemon ${APPS_DIR}

    cd ${PROJECT_TOP}
    return 0
}

# function to build ebtables utility
function build_ebtables() {
    echo "Build ebtables"
    make -w -C ${EBTABLES_DIR} CROSS=${CROSS_COMPILE} KERNEL_INCLUDES=${TURNKEY_KERNEL_DIR}/include

    cd ${PROJECT_TOP}
    return 0
}

# function to install ebtables utility
function install_ebtables() {
    echo "Install ebtables"
    cp -f ${EBTABLES_DIR}/ebtables ${APPS_DIR}
    cp -f ${EBTABLES_DIR}/ethertypes ${ROOTFS_DIR}/rootfs/etc

    cd ${PROJECT_TOP}
    return 0
}

# function to build udhcpc utility
function build_udhcpc() {
    echo "Build udhcpc"
    make -w -C ${UDHCP_DIR} CC=${CROSS_COMPILE}gcc udhcpc

    cd ${PROJECT_TOP}
    return 0
}

# function to install udhcpc utility
function install_udhcpc() {
    echo "Install udhcpc"
    cp -f ${UDHCP_DIR}/udhcpc ${APPS_DIR}

    cd ${ROOTFS_DIR}/rootfs/sbin
    ln -s /apps/udhcpc udhcpc

    cd ${PROJECT_TOP}
    return 0
}

# function to clean udhcpc utility
function clean_udhcpc() {
    echo "Clean udhcpc"
    make -C ${UDHCP_DIR} clean

    cd ${PROJECT_TOP}
    return 0
}

function reconfig_turnkey() {
    echo "Reconfig Turnkey for new function"
    cd ${TURNKEY_TOP}
    
  
    make menuconfig
    
    cd ${PROJECT_TOP}
    return 0
}

function config_turnkey() {
    echo "Config Turnkey before build"
    cd ${TURNKEY_TOP}
    
    if [ ! -d ${TURNKEY_TOP}/romfs/dev ]; then
	mkdir -p romfs
	cd romfs
	echo "Enter root password"
	sudo -u root tar xzvf ../dev.tgz
	cd ${TURNKEY_TOP}
    fi
    
    if [ ! -f ${TURNKEY_TOP}/.config -o "$1" = "default" ]; then
    	echo "Apply Turnkey Default Config"
        cp ${PROJECT_TOP}/config/config ${TURNKEY_TOP}/.config
        cp ${PROJECT_TOP}/config/config_kernel ${TURNKEY_TOP}/linux-2.6.x/.config
        cp ${PROJECT_TOP}/config/config_user ${TURNKEY_TOP}/config/.config
    fi
    
    if [ ! -f ${TURNKEY_TOP}/autoconf.h -o "$1" = "default" ]; then
        cd ${TURNKEY_TOP}
        make menuconfig
    fi
    
    cd ${PROJECT_TOP}
    return 0
}

function apply_turnkey_patch() {
    echo "Apply Turnkey Patches"
    cd ${TURNKEY_TOP}
    
    if [ -f ${TURNKEY_TOP}/patches/patch.sh ]; then
    	${TURNKEY_TOP}/patches/patch.sh
    fi    
    
    cd ${PROJECT_TOP}
    
    if [ -f patches/vendor/patch.sh ]; then
        cd patches/vendor
    	/bin/bash patch.sh
    	cd ${PROJECT_TOP}
    fi
    
    return 0
}

function apply_turnkey_nfs_patch() {
    echo "Apply Turnkey Patches"
    cd ${PROJECT_TOP}
    
	if [ "X${NFS_HOST_IPADDR}" = "X" ]; then
	    echo "please add the setting in "USERCONFIG" to your host ip address for nfsroot "
	    echo 
	    echo "NFS_HOST_IPADDR=yournfsaddress"
	    exit 1
	fi    

	if [ "X${NFS_KERNEL_IPADDR}" = "X" ]; then
	    echo "please add the setting in "USERCONFIG" to your ip address for kenrel nfsroot "
	    echo 
	    echo "NFS_KERNEL_IPADDR=yourkerneladdress"
	    exit 1
	fi
	    
    export NFS_HOST_IPADDR
    export NFS_KERNEL_IPADDR
    
    if [ -f patches/nfs/patch.sh ]; then
        cd patches/nfs
    	/bin/bash patch.sh
    	cd ${PROJECT_TOP}
    else
        echo "ERROR!!: patches/nfs/patch.sh is not exist!"
    fi
    
    return 0
}


function build_turnkey_image() {
    echo "Build Turnkey Images"
    cd ${TURNKEY_KERNEL_DIR}
    # add for custom upgrade verify
    echo "CustomFlag:"$CustomFlag
    if [ -z  "$CustomFlag" ] ; then
         CustomFlag=1
    fi
    ./mkrom.sh $CustomFlag
    ./mkbin/mkbin.sh
    
    if [ ! -f vm.img ]; then    
        echo "Turnkey Build Images Failed"
        return 1
    fi
    
    cp -af ${TURNKEY_KERNEL_DIR}/vm.img ${IMAGE_DIR}/${FS_KERNEL_IMAGE_NAME}-${DATE}
    
    echo "Turnkey Images (Kernel + RootFS) Successfully Built as ${IMAGE_DIR}/${FS_KERNEL_IMAGE_NAME}-${DATE}"
    echo "And copied to ${TFTPROOT} and rename to kernel_fs.img"
    
    #/var/lib/tftpboot is the default root for ubuntu in.tftpd
    if [ ! -d /tftpboot ]; then
        sudo ln -s /var/lib/tftpboot /tftpboot
        sudo chmod 777 /tftpboot
    fi    
    
    if [ -d /tftpboot ]; then
        cp -af ${IMAGE_DIR}/${FS_KERNEL_IMAGE_NAME}-${DATE} ${TFTPROOT}/kernel_fs.img
    else
        echo "/tftpboot is not exist, please make sure the tftp server was installed, and add/link /tftpboot to the root of tftp server"  
    fi
        
    cd ${PROJECT_TOP}
    return 0
}

function download_turnkey() {
    echo "Download Turnkey.."
    
    if [ ! -d ${TURNKEY_TOP} ]; then    
        cd ${VENDOR_DIR}
        mkdir -p ${TURNKEY}
		svn co ${TURNKEY_SRC_URL} ${TURNKEY_TOP}      
    else    
        echo "${TURNKEY_TOP} already exist."
		echo "You have to update the turnkey."
        return 0
    fi

    cd ${PROJECT_TOP}
    
    return 0
}

function update_turnkey() {
    echo "Update Turnkey.."

    if [ -d ${TURNKEY_TOP} ]; then
        cd ${TURNKEY_TOP}
        svn update
    else
        echo "${TURNKEY_TOP} don't exist."
        echo "You have to download the turnkey before."
        return 1
    fi

    return 0
}

#
# Check paramter
#
if [ -z $1 ]; then
    echo "Usage:"
    echo "    build_project.sh [PARAMETER]"
    echo
    echo "PARAMETER:"
    echo "    download:    download Turnkey bootcode, rootfs and kernel from svn"
    echo "    turnkey:     build Turnkey bootcode, rootfs and kernel"
    echo "    turnkey-user:     build Turnkey rootfs only"    
    echo "    turnkey-config: make menuconfig"
    echo "    bootcode:    build Turnkey bootcode"
    echo "    kernel:      build Turnkey rootfs and kernel"
    echo "    rootfs:      build root filesystem"
    echo "    modules:     install Turnkey modules"
    echo "    apps:        build applications"
    echo "    images:      build all images"
    echo "    update:	   update turnkey from svn"
    echo
    echo "Build Sequence:"
    echo "    bootcode -> kernel -> rootfs -> modules -> apps -> images"

    exit 1
fi


#if [ "X${FTPUSER}" = "X" ]; then
#    echo "please change the setting in "USERCONFIG" to your user/passwd for turnkey ftp(201) download"
#    echo 
#    echo "FTPUSER=yourname"
#    echo "FTPPASS=yourpassword"
#    exit 1
#fi

echo "Welcome ${FTPUSER}" 
echo "Please enjoy the ${MODEL_NAME} build script." 


input=$1
choice=$input
option="$2"


case $choice in
    download)
        download_turnkey
        exit 0
        ;;
    bootcode)
        build_bootloader
        exit 0
        ;;

    bootcode-clean)
        clean_bootloader
        exit 0
        ;;

    kernel)
        build_fs_kernel
        exit 0
        ;;

    kernel-clean)
        clean_fs_kernel
        exit 0
        ;;

    turnkey)
        config_turnkey
        #build_bootloader
	# first time, prepare the size of vmbin.img
	# reset the size
	
        build_fs_kernel
        build_turnkey_image
	if [ "$DO_TWO_TIME_KER" = "yes" ]; then
		LEN_CHECKSUM_CAL_ENV=`wc -c ${TURNKEY_KERNEL_DIR}/vmbin.img | awk '{ print $1 }'`
		if [ -z "$LEN_CHECKSUM_CAL_ENV" ]; then
			echo "LEN_CHECKSUM_CAL_ENV cal error, check please";
			exit 1;
		fi
		export LEN_CHECKSUM_CAL_ENV
		# final image/bin file
		build_fs_kernel
		build_turnkey_image
		FINAL_LEN_CHECKSUM_CAL_ENV=`wc -c ${TURNKEY_KERNEL_DIR}/vmbin.img | awk '{ print $1 }'`
		if [ -z "$FINAL_LEN_CHECKSUM_CAL_ENV" ]; then
			echo "FINAL_LEN_CHECKSUM_CAL_ENV cal error, check please";
			exit 1;
		fi
		if [ "$FINAL_LEN_CHECKSUM_CAL_ENV" != "$LEN_CHECKSUM_CAL_ENV" ]; then
			echo "crit: the vmbin.img do not have the same size, please correct it first, else checksum will be fail in bin/img"
			echo "info: the pre size:$LEN_CHECKSUM_CAL_ENV, new size: $FINAL_LEN_CHECKSUM_CAL_ENV"
			exit 1;
		else
			echo "good news: the vmbin.img have the same size($LEN_CHECKSUM_CAL_ENV)"
		fi
	fi
        exit 0
        ;;
        
    turnkey-defconfig)
        config_turnkey default
        clean_turnkey_boa_LINUX
        exit 0
        ;;        
        
    turnkey-user)
        config_turnkey
        #build_bootloader
        build_fs
        #build_turnkey_image
        exit 0
        ;;        
        
    turnkey-patch)
        apply_turnkey_patch
        exit 0
        ;;
        
    turnkey-nfsroot-patch)
        apply_turnkey_nfs_patch
        exit 0
        ;;         
        
    turnkey-clean)
        clean_bootloader
        clean_fs_kernel
	clean_fs_romfs
        exit 0
        ;;
        
    turnkey-config)
        reconfig_turnkey
        exit 0
        ;;
    update)
        update_turnkey
        exit 0
        ;;
esac

echo "Build option is not supported!"

# Below case are supported yet, 
case $2 in
    rootfs)
        build_rootfs
        build_busybox

        install_library
        build_share_library
        install_share_library
        exit 0
        ;;

    modules)
        install_vendor_modules
        install_vendor_apps
        exit 0
        ;;

    apps)
        echo "Make other application and utility, then install to appropriate path"
        # install iptables
        build_iptables
        install_iptables

        # install autover
        build_autover
        install_autover

        # install tr069 agent
        build_TR_069_AGENT
        install_TR_069_AGENT

        # install WEB
        build_web_server
        install_web_server

        # install PPPOE
        build_pppoe_server
        install_pppoe_server

        # install updatedd
        build_DDNS_utility
        install_DDNS_utility

        # install ssmtp
        build_SSMTP_utility
        install_SSMTP_utility

        # install ntpclient
        build_NTPCLIENT_utility
        install_NTPCLIENT_utility

        # install upnpd
        build_UPNP_utility
        install_UPNP_utility

        # install mtd-util
        build_MTD_utility
        install_MTD_utility

        # install dnsmasq
        build_dnsmasq_utility
        install_dnsmasq_utility

        # install igmpproxy
        build_igmpproxy
        install_igmpproxy_v3

        # install udaemon
        build_udaemon
        install_udaemon

        # install ebtables
        build_ebtables
        install_ebtables

        # install udhcpc
#        build_udhcpc
#        install_udhcpc

        exit 0
        ;;

    images)
        echo "Copy all application to ${ROOTFS_DIR}/rootfs/apps"
        cp -fR ${APPS_DIR}/* ${ROOTFS_DIR}/rootfs/apps
        chmod +x ${ROOTFS_DIR}/rootfs/apps/*

        echo "Build root file system image"
        cd ${ROOTFS_DIR}
        ${TURNKEY_HOSTTOOLS_DIR}/fakeroot/fakeroot ./buildFS

        echo "Build all images"
        cd ${ROOTFS_DIR}
        cp -f ${TURNKEY_KERNEL_DIR}/vmlinux .

        ${STRIP} --remove-section=.note --remove-section=.comment vmlinux
        ${OBJCOPY} -O binary vmlinux vmlinux.bin
        ${TURNKEY_HOSTTOOLS_DIR}/cmplzma -k -2 vmlinux vmlinux.bin vmlinux.lz
        ${TURNKEY_HOSTTOOLS_DIR}/bcmImageBuilder --output ${FS_KERNEL_IMAGE_NAME} --chip ${TURNKEY_CHIP} --board ${TURNKEY_BOARD_ID} --blocksize ${TURNKEY_FLASHBLK_SIZE} --bootcodefile ${bootcode_FILE} --rootfsfile rootfs.img --kernelfile vmlinux.lz
        ${TURNKEY_HOSTTOOLS_DIR}/bcmImageBuilder --output ${bootcode_FS_KERNEL_IMAGE_NAME} --chip ${TURNKEY_CHIP} --board ${TURNKEY_BOARD_ID} --blocksize ${TURNKEY_FLASHBLK_SIZE} --bootcodefile ${bootcode_FILE} --rootfsfile rootfs.img --kernelfile vmlinux.lz --include-bootcode
        ${TURNKEY_HOSTTOOLS_DIR}/createimg --boardid=${TURNKEY_BOARD_ID} --numbermac=${TURNKEY_NUM_MAC_ADDRESSES} --macaddr=${TURNKEY_BASE_MAC_ADDRESS} --tp=${TURNKEY_MAIN_TP_NUM} --psisize=${TURNKEY_PSI_SIZE} --gponsn=${TURNKEY_GPON_SERIAL_NUMBER} --gponpw=${TURNKEY_GPON_PASSWORD} --inputfile=${bootcode_FS_KERNEL_IMAGE_NAME} --outputfile=${FLASH_IMAGE_NAME}
        ${TURNKEY_HOSTTOOLS_DIR}/addvtoken ${FLASH_IMAGE_NAME} ${FLASH_IMAGE_NAME}.w

        echo "+++++ Add firmware upgrade header +++++"
        ${NAS_DIR}/head_src/head ${ROOTFS_DIR}/${FS_KERNEL_IMAGE_NAME} application ${ROOTFS_DIR}/${FS_KERNEL_IMAGE_NAME}.hdr

        echo "+++++ Copy image to ${IMAGE_DIR} +++++"
        cp -af ${ROOTFS_DIR}/${FS_KERNEL_IMAGE_NAME} ${IMAGE_DIR}/${FS_KERNEL_IMAGE_NAME}-${DATE}
        cp -af ${ROOTFS_DIR}/${FLASH_IMAGE_NAME}.w ${IMAGE_DIR}
        cp -af ${ROOTFS_DIR}/${FS_KERNEL_IMAGE_NAME}.hdr ${IMAGE_DIR}
        echo "Done! ${MODEL_NAME} images has been built in ${IMAGE_DIR}."

        echo "+++++ Copy fs&kernel image to ${TFTPROOT} +++++"
        cp -af ${ROOTFS_DIR}/${bootcode_FS_KERNEL_IMAGE_NAME} ${TFTPROOT}
        cp -af ${ROOTFS_DIR}/${FS_KERNEL_IMAGE_NAME} ${TFTPROOT}
        cp -af ${ROOTFS_DIR}/${FS_KERNEL_IMAGE_NAME}.hdr ${TFTPROOT}

        echo "----- Clean ${ROOTFS_DIR}/rootfs/apps -----"
        rm -rf ${ROOTFS_DIR}/rootfs/apps/*
        exit 0
        ;;

#
# Make/Test individual function for
#
root_fs)
    build_rootfs
    exit 0
    ;;

clean-root_fs)
    clean_rootfs
    exit 0
    ;;

busybox)
    build_busybox
    exit 0
    ;;

library)
    install_library
    exit 0
    ;;

sharelib)
    build_share_library
    install_share_library
    exit 0
    ;;

iptables)
    build_iptables
    install_iptables
    exit 0
    ;;

autover)
    build_autover
    install_autover
    exit 0
    ;;

tr069)
    build_TR_069_AGENT
    install_TR_069_AGENT
    exit 0
    ;;

web_server)
    build_web_server
    install_web_server
    exit 0
    ;;

pppoe_server)
    build_pppoe_server
    install_pppoe_server
    exit 0
    ;;

#pptp_server)
#    build_pptp_server
#    install_pptp_server
#    exit 0
#    ;;

#l2tp_server)
#    build_l2tp
#    install_l2tp
#    exit 0
#    ;;

#ifInit)
#    build_ifInit
#    install_ifInit
#    exit 0
#    ;;

ddns)
    build_DDNS_utility
    install_DDNS_utility
    exit 0
    ;;

ssmtp)
    build_SSMTP_utility
    install_SSMTP_utility
    exit 0
    ;;

ntpclient)
    build_NTPCLIENT_utility
    install_NTPCLIENT_utility
    exit 0
    ;;

upnpd)
    build_UPNP_utility
    install_UPNP_utility
    exit 0
    ;;

mtd_utility)
    build_MTD_utility
    install_MTD_utility
    exit 0
    ;;

dnsmasq)
    build_dnsmasq_utility
    install_dnsmasq_utility
    exit 0
    ;;

igmp_proxy)
    build_igmpproxy
#    install_igmpproxy
    install_igmpproxy_v3
    exit 0
    ;;

udaemon)
    build_udaemon
    install_udaemon
    exit 0
    ;;

ebtables)
    build_ebtables
    install_ebtables
    exit 0
    ;;

udhcpc)
    build_udhcpc
    install_udhcpc
    ;;

clean_udhcpc)
    clean_udhcpc
    ;;   
esac

