/*
 *      Adsl Driver Interface
 *      Authors: Dick Tam	<dicktam@realtek.com.tw>
 *
 */

#include <string.h>
#include <stdlib.h>
#include <time.h>

// from hrchen adslctrl.c
#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
#include <errno.h>
#include <sys/types.h>
// from hrchen adslctrl.c end

//for RLCM_GET_DS_PMS_PARAM1 & RLCM_GET_US_PMS_PARAM1
typedef struct {
	unsigned short	K;
	unsigned short	R;
	unsigned short	D;
	unsigned short	N_fec;
	unsigned short	Lp;
	unsigned short	S;
	unsigned short	Delay;
} Modem_PMSParm;

//for RLCM_GET_DS_ERROR_COUNT & RLCM_GET_US_ERROR_COUNT
typedef struct {
	unsigned long	crc;
	unsigned long	fec;
	unsigned long	es;
	unsigned long	ses;
	unsigned long	uas;
} Modem_MgmtCounter;

//for RLCM_GET_LINE_RATE
typedef struct {
	unsigned short	upstreamRate;
	unsigned short	downstreamRate;
} Modem_LineRate;

//for RLCM_GET_ATT_RATE
typedef struct {
	unsigned short	upstreamRate;
	unsigned short	downstreamRate;
} Modem_AttRate;

#include "adsl_drv.h"
#include "utility.h"
#include "mib.h"

static char rValueAdslDrv[ADSL_DRV_RETURN_LEN];
static char rGetFail[] = "n/a";

adsl_drv_get_cmd get_cmd_list[] = {
	{"snr-ds", ADSL_GET_SNR_DS},
	{"snr-us", ADSL_GET_SNR_US},
	{"version", ADSL_GET_VERSION},
	{"mode", ADSL_GET_MODE},
	{"state", ADSL_GET_STATE},
	{"power-ds", ADSL_GET_POWER_DS},
	{"power-us", ADSL_GET_POWER_US},
	{"attrate-ds", ADSL_GET_ATTRATE_DS},
	{"attrate-us", ADSL_GET_ATTRATE_US},
	{"rate-ds", ADSL_GET_RATE_DS},
	{"rate-us", ADSL_GET_RATE_US},
	{"latency", ADSL_GET_LATENCY},
	{"lpatt-ds", ADSL_GET_LPATT_DS},
	{"lpatt-us", ADSL_GET_LPATT_US},
	{"trellis", ADSL_GET_TRELLIS},
	{"pwlevel", ADSL_GET_POWER_LEVEL},
	{"pms-k-ds", ADSL_GET_K_DS},
	{"pms-k-us", ADSL_GET_K_US},
	{"pms-r-ds", ADSL_GET_R_DS},
	{"pms-r-us", ADSL_GET_R_US},
	{"pms-s-ds", ADSL_GET_S_DS},
	{"pms-s-us", ADSL_GET_S_US},
	{"pms-d-ds", ADSL_GET_D_DS},
	{"pms-d-us", ADSL_GET_D_US},
	{"pms-delay-ds", ADSL_GET_DELAY_DS},
	{"pms-delay-us", ADSL_GET_DELAY_US},
	{"fec-ds", ADSL_GET_FEC_DS},
	{"fec-us", ADSL_GET_FEC_US},
	{"crc-ds", ADSL_GET_CRC_DS},
	{"crc-us", ADSL_GET_CRC_US},
	{"es-ds", ADSL_GET_ES_DS},
	{"es-us", ADSL_GET_ES_US},
	{"ses-ds", ADSL_GET_SES_DS},
	{"ses-us", ADSL_GET_SES_US},
	{"uas-ds", ADSL_GET_UAS_DS},
	{"uas-us", ADSL_GET_UAS_US},
	//add by liuxiao 2008-01-25
	{"downRate", ADSL_GET_RATE_DS},
	{"upRate", ADSL_GET_RATE_US},
	{"syncState", ADSL_GET_STATE},
	//add end by liuxiao 2008-01-25
	//add by liuxiao 2008-01-26
	{"pmsLatency", ADSL_GET_LATENCY},
	//add end by liuxiao 2008-01-26
	{NULL, 0}
};

/*
 *	Get adsl string according to the id.
 *	The string is put into buf, and this function return the number of
 *	characters in the string (not including the trailing '\0')
 */
int getAdslInfo(ADSL_GET_ID id, char *buf, int len)
{
	int ret;
	int intVal[2];
	char chVal[2];
	Modem_NearEndLineOperData vNeld;
	Modem_FarEndLineOperData vFeld;
	Modem_AttRate vAr;
	Modem_LinkSpeed vLs;
	Modem_PMSParm pms;
	Modem_AvgLoopAttenuation vAla;
	Modem_Config vMc;
	Modem_MgmtCounter counter;
	
	ret = 0;
	
	switch (id) {
		case ADSL_GET_SNR_DS:
			if(adsl_drv_get(RLCM_GET_SNR_MARGIN, intVal, RLCM_GET_SNR_MARGIN_SIZE)) {
				ret = snprintf(buf, len, "%d.%d", intVal[0]/10, intVal[0]%10);
			}
			break;
		case ADSL_GET_SNR_US:
			if(adsl_drv_get(RLCM_GET_SNR_MARGIN, intVal, RLCM_GET_SNR_MARGIN_SIZE)) {
				ret = snprintf(buf, len, "%d.%d", intVal[1]/10, intVal[1]%10);
			}
			break;
		case ADSL_GET_VERSION:
			if(adsl_drv_get(RLCM_GET_DRIVER_VERSION, (void *)rValueAdslDrv, RLCM_DRIVER_VERSION_SIZE)) {
				ret = snprintf(buf, len, "%s", rValueAdslDrv);
			}
			break;
		case ADSL_GET_MODE:
			if(adsl_drv_get(RLCM_GET_SHOWTIME_XDSL_MODE, (void *)&chVal[0], 1)) {
				// suit for annex mode append to showtime mode...
				unsigned int annexMode;
				char *ExpMode[]=
				{
					{""},
					{" AnnexB"},
					{" AnnexM"},
					{""},
					{" AnnexL"},
					{0}
				};
				annexMode=((unsigned char)chVal[0])>>5;
				if(annexMode >4)
					annexMode=0;
				chVal[0]&=0x1F;
				if (chVal[0] == 1) { // ADSL1
					adsl_drv_get(RLCM_GET_ADSL_MODE, (void *)&intVal[0], 4);
					if (intVal[0]==1)
						ret = snprintf(buf, len, "%s", "T1.413");
					if (intVal[0]==2)
						ret = snprintf(buf, len, "%s", "G.dmt");
				}
				else if (chVal[0] == 2)  // G.992.2 -- G.Lite
						ret = snprintf(buf, len, "%s", "G.Lite");
				else if (chVal[0] == 3)  // ADSL2
						ret = snprintf(buf, len, "%s", "ADSL2");
				else if (chVal[0] == 5)  // ADSL2+
						ret = snprintf(buf, len, "%s", "ADSL2+");
				//printf("annex mode =%s\n", ExpMode[annexMode]);
				strcat(buf, ExpMode[annexMode]);
			}
			break;
		// get string
		case ADSL_GET_STATE:
			if(adsl_drv_get(RLCM_REPORT_MODEM_STATE, (void *)buf, len)) {
				ret = strlen(buf);
			}
			else
				ret = snprintf(buf, len, "%s", "IDLE");
			break;
		case ADSL_GET_POWER_DS:
			if(adsl_drv_get(RLCM_MODEM_FAR_END_LINE_DATA_REQ, (void *)&vFeld, RLCM_MODEM_FAR_END_LINE_DATA_REQ_SIZE)) {
				ret = snprintf(buf, len, "%d.%d" ,vFeld.outputPowerDnstr/2, ((vFeld.outputPowerDnstr&1)*5)%10);
			}
			break;
		case ADSL_GET_POWER_US:
			if(adsl_drv_get(RLCM_MODEM_NEAR_END_LINE_DATA_REQ, (void *)&vNeld, RLCM_MODEM_NEAR_END_LINE_DATA_REQ_SIZE)) {
				ret = snprintf(buf, len, "%d.%d" ,vNeld.outputPowerUpstr/2, ((vNeld.outputPowerUpstr&1)*5)%10);
			}
			break;
		case ADSL_GET_ATTRATE_DS:
			if(adsl_drv_get(RLCM_GET_ATT_RATE, (void *)&vAr, sizeof(Modem_AttRate))) {
				ret = snprintf(buf, len, "%d"
					,vAr.downstreamRate);
			}
			break;
		case ADSL_GET_ATTRATE_US:
			if(adsl_drv_get(RLCM_GET_ATT_RATE, (void *)&vAr, sizeof(Modem_AttRate))) {
				ret = snprintf(buf, len, "%d"
					,vAr.upstreamRate);
			}
			break;
		case ADSL_GET_RATE_DS:
			if(adsl_drv_get(RLCM_GET_LINK_SPEED, (void *)&vLs, RLCM_GET_LINK_SPEED_SIZE)) {
				ret = snprintf(buf, len, "%d"
					,vLs.downstreamRate);
			}
			break;
		case ADSL_GET_RATE_US:
			if(adsl_drv_get(RLCM_GET_LINK_SPEED, (void *)&vLs, RLCM_GET_LINK_SPEED_SIZE)) {
				ret = snprintf(buf, len, "%d"
					,vLs.upstreamRate);
			}
			break;
		case ADSL_GET_LATENCY:
			if(adsl_drv_get(RLCM_GET_DS_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
				if (pms.D==1)
					ret = snprintf(buf, len, "%s", "Fast");
				else
					ret = snprintf(buf, len, "%s", "Interleave");
			}
			break;
		case ADSL_GET_LPATT_DS:
			if(adsl_drv_get(RLCM_GET_LOOP_ATT, (void *)&vAla, RLCM_GET_LOOP_ATT_SIZE)) {
				ret = snprintf(buf, len, "%d.%d", vAla.downstreamAtt/2, (vAla.downstreamAtt*5)%10);
			}
			break;
		case ADSL_GET_LPATT_US:
			if(adsl_drv_get(RLCM_GET_LOOP_ATT, (void *)&vAla, RLCM_GET_LOOP_ATT_SIZE)) {
				ret = snprintf(buf, len, "%d.%d", vAla.upstreamAtt/2, (vAla.upstreamAtt*5)%10);
			}
			break;
		case ADSL_GET_TRELLIS:
			if(adsl_drv_get(RLCM_MODEM_READ_CONFIG, (void *)&vMc, RLCM_MODEM_CONFIG_SIZE)) {
				ret = snprintf(buf, len, "%s", (vMc.TrellisEnable==1)?"Enable":"Disable");
			}
			break;
		case ADSL_GET_POWER_LEVEL:
			if(adsl_drv_get(RLCM_GET_LINK_POWER_STATE, (void *)&chVal[0], 1)) {
				if (chVal[0] == 0)
					ret = snprintf(buf, len, "%s", "L0");
				else if (chVal[0] == 2)
					ret = snprintf(buf, len, "%s", "L2");
				else if (chVal[0] == 3)
					ret = snprintf(buf, len, "%s", "L3");
			}
			break;
		case ADSL_GET_K_DS:
		case ADSL_GET_R_DS:
		case ADSL_GET_S_DS:
		case ADSL_GET_D_DS:
		case ADSL_GET_DELAY_DS:
			memset((void *)&pms, 0, sizeof(pms));
			if(adsl_drv_get(RLCM_GET_DS_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
				switch (id) {
					case ADSL_GET_K_DS:
						ret = snprintf(buf, len, "%d", pms.K);
						break;
					case ADSL_GET_R_DS:
						ret = snprintf(buf, len, "%d", pms.R);
						break;
					case ADSL_GET_S_DS:
						ret = snprintf(buf, len, "%d.%02d", pms.S/100, pms.S%100);
						break;
					case ADSL_GET_D_DS:
						ret = snprintf(buf, len, "%d", pms.D);
						break;
					case ADSL_GET_DELAY_DS:
						ret = snprintf(buf, len, "%d.%02d", pms.Delay/100, pms.Delay%100);
						break;
					default:break;
				}
			}
			break;
		case ADSL_GET_K_US:
		case ADSL_GET_R_US:
		case ADSL_GET_S_US:
		case ADSL_GET_D_US:
		case ADSL_GET_DELAY_US:
			memset((void *)&pms, 0, sizeof(pms));
			if(adsl_drv_get(RLCM_GET_US_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
				switch (id) {
					case ADSL_GET_K_US:
						ret = snprintf(buf, len, "%d", pms.K);
						break;
					case ADSL_GET_R_US:
						ret = snprintf(buf, len, "%d", pms.R);
						break;
					case ADSL_GET_S_US:
						ret = snprintf(buf, len, "%d.%02d", pms.S/100, pms.S%100);
						break;
					case ADSL_GET_D_US:
						ret = snprintf(buf, len, "%d", pms.D);
						break;
					case ADSL_GET_DELAY_US:
						ret = snprintf(buf, len, "%d.%02d", pms.Delay/100, pms.Delay%100);
						break;
					default:break;
				}
			}
			break;
		case ADSL_GET_CRC_DS:
		case ADSL_GET_FEC_DS:
		case ADSL_GET_ES_DS:
		case ADSL_GET_SES_DS:
		case ADSL_GET_UAS_DS:
			memset((void *)&counter, 0, sizeof(counter));
			if(adsl_drv_get(RLCM_GET_DS_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
				unsigned long *cnt;
				cnt = (unsigned long *)&counter;
				ret = snprintf(buf, len, "%d", cnt[id-ADSL_GET_CRC_DS]);
			}
			break;
		case ADSL_GET_CRC_US:
		case ADSL_GET_FEC_US:
		case ADSL_GET_ES_US:
		case ADSL_GET_SES_US:
		case ADSL_GET_UAS_US:
			memset((void *)&counter, 0, sizeof(counter));
			if(adsl_drv_get(RLCM_GET_US_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
				//unsigned long *cnt;
				unsigned short *cnt;	// temporarily, should use long
				cnt = (unsigned short *)&counter;
				ret = snprintf(buf, len, "%d", cnt[id-ADSL_GET_CRC_US]);
			}
			break;
		default:
			break;
	}
	
	return ret;
}
		
		
int getAdslDrvInfo(char *name, char *buff, int len)
{
	int idx, length;
	
	length = 0;
	
	for (idx=0; get_cmd_list[idx].cmd != NULL; idx++) {
		if (!strcmp(name, get_cmd_list[idx].cmd)) {
			length=getAdslInfo(get_cmd_list[idx].id, buff, len);
			break;
		}
	}
	
	if (length == 0)
		length = snprintf(buff, len, "%s", BLANK);
	
	return length;
}

#if 0
char* getAdslDrvInfo(char *name)
{
	int vInt;
	unsigned int vUInt;
	Modem_Identification vMId;
	Modem_NearEndLineOperData vNeld;
	Modem_FarEndLineOperData vFeld;
	Modem_LossData vLd;
	Modem_LinkSpeed vLs;
	Modem_LineRate vLr;
	Modem_AvgLoopAttenuation vAla;
	Modem_Config vMc;
	Modem_PMSParm pms;
	char *ret;
	
	ret = rValueAdslDrv;
//	strcpy(rValueAdslDrv,"ADSL DRV: Unknown Value\n");

	if ( !strcmp(name, "adsl-drv-version")) {
		rValueAdslDrv[RLCM_DRIVER_VERSION_SIZE] = 0;
		if(!adsl_drv_get(RLCM_GET_DRIVER_VERSION, (void *)rValueAdslDrv, RLCM_DRIVER_VERSION_SIZE)) {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	} 
	else if ( !strcmp(name, "adsl-drv-build")) {
		rValueAdslDrv[RLCM_DRIVER_BUILD_SIZE] = 0;
		if(!adsl_drv_get(RLCM_GET_DRIVER_BUILD, (void *)rValueAdslDrv, RLCM_DRIVER_BUILD_SIZE)) {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	} 
	else if ( !strcmp(name, "adsl-drv-rehskcount")) {
		if(adsl_drv_get(RLCM_GET_REHS_COUNT, (void *)&vInt, RLCM_GET_REHS_COUNT_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	} 
	else if ( !strcmp(name, "adsl-drv-avgsnr")) {
		if(adsl_drv_get(RLCM_GET_AVERAGE_SNR, (void *)&vInt, RLCM_GET_AVERAGE_SNR_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d dB", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-snrdown")) {
		int snrMargin[2];
		if(adsl_drv_get(RLCM_GET_SNR_MARGIN, snrMargin, RLCM_GET_SNR_MARGIN_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", snrMargin[0]);
		} else {
			ret = rGetFail;
		}
	}
	else if ( !strcmp(name, "adsl-drv-snrup")) {
		int snrMargin[2];
		if(adsl_drv_get(RLCM_GET_SNR_MARGIN, snrMargin, RLCM_GET_SNR_MARGIN_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", snrMargin[1]);
		} else {
			ret = rGetFail;
		}
	}
	else if ( !strcmp(name, "adsl-drv-snrmargin")) {
		int snrMargin[2];
		if(adsl_drv_get(RLCM_GET_SNR_MARGIN, snrMargin, RLCM_GET_SNR_MARGIN_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "Down: %d dB, Up: %d dB", snrMargin[0], snrMargin[1]);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-mode")) {
		char mode=0;
		rValueAdslDrv[64] = rValueAdslDrv[0] = 0;
		if(adsl_drv_get(RLCM_GET_SHOWTIME_XDSL_MODE, (void *)&mode, 1)) {
			if (mode == 1) { // ADSL1
				adsl_drv_get(RLCM_GET_ADSL_MODE, (void *)&vInt, 4);
				if (vInt==1)
					strcpy(rValueAdslDrv, "T1.413");
				if (vInt==2)
					strcpy(rValueAdslDrv, "G.dmt");
			}
			else if (mode == 3)  // ADSL2
					strcpy(rValueAdslDrv, "ADSL2");
			else if (mode == 5)  // ADSL2+
					strcpy(rValueAdslDrv, "ADSL2+");
			else
				ret = rGetFail;
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-modemstate")) {
		if(!adsl_drv_get(RLCM_REPORT_MODEM_STATE, (void *)rValueAdslDrv, 64)) {
			strcpy(rValueAdslDrv,"IDLE\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-modemstatus")) {
		char modeStr[16], statusStr[64];
		rValueAdslDrv[64] = 0;
		if(adsl_drv_get(RLCM_GET_ADSL_MODE, (void *)&vInt, 4)) {
			if (vInt==1)
				strcpy(modeStr, "T1.413");
			if (vInt==2)
				strcpy(modeStr, "G.dmt");
			if (vInt==3)
				strcpy(modeStr, "MultiMode");
		} else {
			strcpy(modeStr, "unknown mode");
		}
		if(adsl_drv_get(RLCM_REPORT_MODEM_STATE, (void *)statusStr, 64)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%s, %s"
				,modeStr
				,statusStr);
		} else {
			strcpy(rValueAdslDrv,"IDLE\n");
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-pmdata")) {
		Modem_def_counter_set vCs;
		if(adsl_drv_get(RLCM_REPORT_PM_DATA, (void *)&vCs, RLCM_REPORT_PM_DATA_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "NE(%u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u) FE(%u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u, %u)"
				,vCs.near_end.FecNotInterleaved
				,vCs.near_end.FecInterleaved
				,vCs.near_end.CrcNotInterleaved
				,vCs.near_end.CrcInterleaved
				,vCs.near_end.HecNotInterleaved
				,vCs.near_end.HecInterleaved
				,vCs.near_end.TotalCellCountInterleaved
				,vCs.near_end.TotalCellCountNotInterleaved
				,vCs.near_end.ActiveCellCountInterleaved
				,vCs.near_end.ActiveCellCountNotInterleaved
				,vCs.near_end.BERInterleaved
				,vCs.near_end.BERNotInterleaved
				,vCs.far_end.FecNotInterleaved
				,vCs.far_end.FecInterleaved
				,vCs.far_end.CrcNotInterleaved
				,vCs.far_end.CrcInterleaved
				,vCs.far_end.HecNotInterleaved
				,vCs.far_end.HecInterleaved
				,vCs.far_end.TotalCellCountInterleaved
				,vCs.far_end.TotalCellCountNotInterleaved
				,vCs.far_end.ActiveCellCountInterleaved
				,vCs.far_end.ActiveCellCountNotInterleaved
				,vCs.far_end.BERInterleaved
				,vCs.far_end.BERNotInterleaved);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-neid")) {
		if(adsl_drv_get(RLCM_MODEM_NEAR_END_ID_REQ, (void *)&vMId, RLCM_MODEM_ID_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "ITU Vendor Id(%02X, %02X, %4s, %04X), Std Revision(%02X) ESTI Vendor Id(%04X), Vendor Revision(%02X), Std Revision(%02X), Mmgt Info(%08lX)"
				,vMId.ITU_VendorId.countryCode
				,vMId.ITU_VendorId.reserved
				,(char *)&(vMId.ITU_VendorId.vendorCode)
				,vMId.ITU_VendorId.vendorSpecific
				,vMId.ITU_StandardRevisionNbr
				,vMId.ANSI_ETSI_VendorId
				,vMId.ANSI_ETSI_VendorRevisionNbr
				,vMId.ANSI_ETSI_StandardRevisionNbr
				,vMId.ALC_ManagementInfo);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-feid")) {
		if(adsl_drv_get(RLCM_MODEM_FAR_END_ID_REQ, (void *)&vMId, RLCM_MODEM_ID_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "ITU Vendor Id(%02X, %02X, %4s, %04X), Std Revision(%02X) ESTI Vendor Id(%04X), Vendor Revision(%02X), Std Revision(%02X), Mmgt Info(%08lX)"
				,vMId.ITU_VendorId.countryCode
				,vMId.ITU_VendorId.reserved
				,(char *)&(vMId.ITU_VendorId.vendorCode)
				,vMId.ITU_VendorId.vendorSpecific
				,vMId.ITU_StandardRevisionNbr
				,vMId.ANSI_ETSI_VendorId
				,vMId.ANSI_ETSI_VendorRevisionNbr
				,vMId.ANSI_ETSI_StandardRevisionNbr
				,vMId.ALC_ManagementInfo);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-power-us")) {
		if(adsl_drv_get(RLCM_MODEM_NEAR_END_LINE_DATA_REQ, (void *)&vNeld, RLCM_MODEM_NEAR_END_LINE_DATA_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d" ,vNeld.outputPowerUpstr);
		} else {
			ret = rGetFail;
		}
	}
	else if ( !strcmp(name, "adsl-drv-power-ds")) {
		if(adsl_drv_get(RLCM_MODEM_FAR_END_LINE_DATA_REQ, (void *)&vFeld, RLCM_MODEM_FAR_END_LINE_DATA_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d" ,vFeld.outputPowerDnstr);
		} else {
			ret = rGetFail;
		}
	}
	else if ( !strcmp(name, "adsl-drv-neld")) {
		if(adsl_drv_get(RLCM_MODEM_NEAR_END_LINE_DATA_REQ, (void *)&vNeld, RLCM_MODEM_NEAR_END_LINE_DATA_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "relCapacityOccupationDnstr=%u, noiseMarginDnstr=%d, outputPowerUpstr=%d, attenuationDnstr=%u, operationalMode=%08lX"
				,vNeld.relCapacityOccupationDnstr
				,vNeld.noiseMarginDnstr
				,vNeld.outputPowerUpstr
				,vNeld.attenuationDnstr
				,vNeld.operationalMode);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-feld")) {
		if(adsl_drv_get(RLCM_MODEM_FAR_END_LINE_DATA_REQ, (void *)&vFeld, RLCM_MODEM_FAR_END_LINE_DATA_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "relCapacityOccupationUpstr=%u, noiseMarginUpstr=%d, outputPowerDnstr=%d, attenuationUpstr=%u"
				,vFeld.relCapacityOccupationUpstr
				,vFeld.noiseMarginUpstr
				,vFeld.outputPowerDnstr
				,vFeld.attenuationUpstr);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-necdfm")) {
		if(adsl_drv_get(RLCM_MODEM_NEAR_END_FAST_CH_DATA_REQ, (void *)&vInt, RLCM_MODEM_CH_DATA_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-necdim")) {
		if(adsl_drv_get(RLCM_MODEM_NEAR_END_INT_CH_DATA_REQ, (void *)&vInt, RLCM_MODEM_CH_DATA_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-fecdfm")) {
		if(adsl_drv_get(RLCM_MODEM_FAR_END_FAST_CH_DATA_REQ, (void *)&vInt, RLCM_MODEM_CH_DATA_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-fecdim")) {
		if(adsl_drv_get(RLCM_MODEM_FAR_END_INT_CH_DATA_REQ, (void *)&vInt, RLCM_MODEM_CH_DATA_REQ_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-lossdata")) {
		if(adsl_drv_get(RLCM_GET_LOSS_DATA, (void *)&vLd, RLCM_GET_LOSS_DATA_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "LOF NE(%08lX), LOF FE(%08lX), LOS NE(%08lX), LOS FE(%08lX), LOP NE(%08lX), LOP FE(%08lX), NELCD F(%08lX), FELCD F(%08lX), NELCD I(%08lX), FELCD I(%08lX), NECRC F(%08lX), NECRC I(%08lX), NEESs(%08lX)"
				,vLd.LossOfFrame_NE
				,vLd.LossOfFrame_FE
				,vLd.LossOfSignal_NE
				,vLd.LossOfSignal_FE
				,vLd.LossOfPower_NE
				,vLd.LossOfPower_FE
				,vLd.LCD_Fast_NE
				,vLd.LCD_Fast_FE
				,vLd.LCD_Interleaved_NE
				,vLd.LCD_Interleaved_FE
				,vLd.CrcCounter_Fast_NE
				,vLd.CrcCounter_Interleaved_NE
				,vLd.ESs_NE);
			
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-15min-lossdata")) {
		if(adsl_drv_get(RLCM_GET_15MIN_LOSS_DATA, (void *)&vLd, RLCM_GET_LOSS_DATA_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "LOF NE(%08lX), LOF FE(%08lX), LOS NE(%08lX), LOS FE(%08lX), LOP NE(%08lX), LOP FE(%08lX), NELCD F(%08lX), FELCD F(%08lX), NELCD I(%08lX), FELCD I(%08lX), NECRC F(%08lX), NECRC I(%08lX), NEESs(%08lX)"
				,vLd.LossOfFrame_NE
				,vLd.LossOfFrame_FE
				,vLd.LossOfSignal_NE
				,vLd.LossOfSignal_FE
				,vLd.LossOfPower_NE
				,vLd.LossOfPower_FE
				,vLd.LCD_Fast_NE
				,vLd.LCD_Fast_FE
				,vLd.LCD_Interleaved_NE
				,vLd.LCD_Interleaved_FE
				,vLd.CrcCounter_Fast_NE
				,vLd.CrcCounter_Interleaved_NE
				,vLd.ESs_NE);
			
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-1day-lossdata")) {
		if(adsl_drv_get(RLCM_GET_1DAY_LOSS_DATA, (void *)&vLd, RLCM_GET_LOSS_DATA_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "LOF NE(%08lX), LOF FE(%08lX), LOS NE(%08lX), LOS FE(%08lX), LOP NE(%08lX), LOP FE(%08lX), NELCD F(%08lX), FELCD F(%08lX), NELCD I(%08lX), FELCD I(%08lX), NECRC F(%08lX), NECRC I(%08lX), NEESs(%08lX)"
				,vLd.LossOfFrame_NE
				,vLd.LossOfFrame_FE
				,vLd.LossOfSignal_NE
				,vLd.LossOfSignal_FE
				,vLd.LossOfPower_NE
				,vLd.LossOfPower_FE
				,vLd.LCD_Fast_NE
				,vLd.LCD_Fast_FE
				,vLd.LCD_Interleaved_NE
				,vLd.LCD_Interleaved_FE
				,vLd.CrcCounter_Fast_NE
				,vLd.CrcCounter_Interleaved_NE
				,vLd.ESs_NE);
			
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-attuprate")) {
		if(adsl_drv_get(RLCM_GET_LINE_RATE, (void *)&vLr, sizeof(Modem_LineRate))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d"
				,vLr.upstreamRate);
		} else {
			strcpy(rValueAdslDrv,"0");
		}
	}
	else if ( !strcmp(name, "adsl-drv-attdownrate")) {
		if(adsl_drv_get(RLCM_GET_LINE_RATE, (void *)&vLr, sizeof(Modem_LineRate))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d"
				,vLr.downstreamRate);
		} else {
			strcpy(rValueAdslDrv,"0");
		}
	}
	else if ( !strcmp(name, "adsl-drv-uprate")) {
		if(adsl_drv_get(RLCM_GET_LINK_SPEED, (void *)&vLs, RLCM_GET_LINK_SPEED_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d"
				,vLs.upstreamRate);
		} else {
			strcpy(rValueAdslDrv,"0");
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-downrate")) {
		if(adsl_drv_get(RLCM_GET_LINK_SPEED, (void *)&vLs, RLCM_GET_LINK_SPEED_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d"
				,vLs.downstreamRate);
		} else {
			strcpy(rValueAdslDrv,"0");
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-latency")) {
		/*
		vInt = 0;
		if(adsl_drv_get(RLCM_GET_CHANNEL_MODE, (void *)&vInt, 4)) {
			if (vInt==1)
				strcpy(rValueAdslDrv, "Fast");
			else if (vInt==2)
				strcpy(rValueAdslDrv, "Interleave");
			else
				ret = rGetFail;
		} else
			ret = rGetFail;
		*/
		if(adsl_drv_get(RLCM_GET_DS_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			if (pms.D==1)
				strcpy(rValueAdslDrv, "Fast");
			else
				strcpy(rValueAdslDrv, "Interleave");
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-linkstatus")) {
		char modeStr[16];
		vInt = 0;
		strcpy(modeStr, "unknown");
		if(adsl_drv_get(RLCM_GET_CHANNEL_MODE, (void *)&vInt, 4)) {
			if (vInt==1)
			    strcpy(modeStr, "Fast");
			if (vInt==2)
			    strcpy(modeStr, "Interleave");
		}
		if(adsl_drv_get(RLCM_GET_LINK_SPEED, (void *)&vLs, RLCM_GET_LINK_SPEED_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "upstreamRate(%d), downstreamRate(%d), dataMode(%s)"
				,vLs.upstreamRate
				,vLs.downstreamRate
				,modeStr);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-channelmode")) {
		if(adsl_drv_get(RLCM_GET_CHANNEL_MODE, (void *)&vInt, 4)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-loopatt-up")) {
		if(adsl_drv_get(RLCM_GET_LOOP_ATT, (void *)&vAla, RLCM_GET_LOOP_ATT_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vAla.upstreamAtt);
		} else {
			ret = rGetFail;
		}
	}
	else if ( !strcmp(name, "adsl-drv-loopatt-down")) {
		if(adsl_drv_get(RLCM_GET_LOOP_ATT, (void *)&vAla, RLCM_GET_LOOP_ATT_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vAla.downstreamAtt);
		} else {
			ret = rGetFail;
		}
	}
	else if ( !strcmp(name, "adsl-drv-loopatt")) {
		if(adsl_drv_get(RLCM_GET_LOOP_ATT, (void *)&vAla, RLCM_GET_LOOP_ATT_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "upstreamAtt(%04x), downstreamAtt(%04x)"
				,vAla.upstreamAtt
				,vAla.downstreamAtt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-txpower")) {
		if(adsl_drv_get(RLCM_GET_TX_POWER, (void *)&vInt, 4)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-tuneper")) {
		if(adsl_drv_get(RLCM_GET_PERF_VALUE, (void *)&vInt, 4)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-bitswap")) {
		if(adsl_drv_get(RLCM_MODEM_READ_CONFIG, (void *)&vMc, RLCM_MODEM_CONFIG_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%s", (vMc.BitSwapEnable==1)?"Enable":"Disable");
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-pilotrelocat")) {
		if(adsl_drv_get(RLCM_MODEM_READ_CONFIG, (void *)&vMc, RLCM_MODEM_CONFIG_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%s", (vMc.PilotRelocationEnable==1)?"Enable":"Disable");
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-trellis")) {
		if(adsl_drv_get(RLCM_MODEM_READ_CONFIG, (void *)&vMc, RLCM_MODEM_CONFIG_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%s", (vMc.TrellisEnable==1)?"Enable":"Disable");
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-vendorid")) {
		if(adsl_drv_get(RLCM_GET_VENDOR_ID, (void *)&vInt, 4)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", vInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-config")) {
		if(adsl_drv_get(RLCM_MODEM_READ_CONFIG, (void *)&vMc, RLCM_MODEM_CONFIG_SIZE)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "HandshakeMode(%d), TxPower(%d), PerfTuning(%d), AtuC_VendorID(%d), BitSwapEnable(%d), PilotRelocationEnable(%d), TrellisEnable(%d)"
				,vMc.HandshakeMode
				,vMc.TxPower
				,vMc.PerfTuning
				,vMc.AtuC_VendorID
				,vMc.BitSwapEnable
				,vMc.PilotRelocationEnable
				,vMc.TrellisEnable);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-adsltime")) {
		if(adsl_drv_get(RLCM_GET_ADSL_TIME, (void *)&vUInt, 4)) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%u", vUInt);
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-power")) {
		char level;
		if(adsl_drv_get(RLCM_GET_LINK_POWER_STATE, (void *)&level, 1)) {
			if (level == 0)
				strcpy(rValueAdslDrv, "L0");
			else if (level == 2)
				strcpy(rValueAdslDrv, "L2");
			else if (level == 3)
				strcpy(rValueAdslDrv, "L3");
			else
				ret = rGetFail;
		} else {
			ret = rGetFail;
//			strcpy(rValueAdslDrv,"ADSL DRV: ioctl fail!\n");
		}
	}
	else if ( !strcmp(name, "adsl-drv-pms-k-ds")) {
		if(adsl_drv_get(RLCM_GET_DS_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", pms.K);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-pms-r-ds")) {
		if(adsl_drv_get(RLCM_GET_DS_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", pms.R);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-pms-s-ds")) {
		if(adsl_drv_get(RLCM_GET_DS_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", 8*pms.N_fec/pms.Lp);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-pms-d-ds")) {
		if(adsl_drv_get(RLCM_GET_DS_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", pms.D);
		}
		else
			ret = rGetFail;
	}
	/*
	else if ( !strcmp(name, "adsl-drv-pms-delay-ds")) {
		if(adsl_drv_get(RLCM_GET_DS_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", (8*pms.N_fec*pms.D)/(pms.Lp*4));
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-pms-k-us")) {
		Modem_PMSParm pms;
		if(adsl_drv_get(RLCM_GET_US_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", pms.K);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-pms-r-us")) {
		Modem_PMSParm pms;
		if(adsl_drv_get(RLCM_GET_US_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", pms.R);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-pms-s-us")) {
		Modem_PMSParm pms;
		if(adsl_drv_get(RLCM_GET_US_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", 8*pms.N_fec/pms.Lp);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-pms-d-us")) {
		Modem_PMSParm pms;
		if(adsl_drv_get(RLCM_GET_US_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", pms.D);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-pms-delay-us")) {
		Modem_PMSParm pms;
		if(adsl_drv_get(RLCM_GET_US_PMS_PARAM1, (void *)&pms, sizeof(pms))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", (8*pms.N_fec*pms.D)/(pms.Lp*4));
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-fec-ds")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_DS_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.fec);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-crc-ds")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_DS_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.crc);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-es-ds")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_DS_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.es);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-ses-ds")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_DS_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.ses);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-uas-ds")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_DS_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.uas);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-fec-us")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_US_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.fec);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-crc-us")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_US_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.crc);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-es-us")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_US_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.es);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-ses-us")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_US_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.ses);
		}
		else
			ret = rGetFail;
	}
	else if ( !strcmp(name, "adsl-drv-uas-us")) {
		Modem_MgmtCounter counter;
		if(adsl_drv_get(RLCM_GET_US_ERROR_COUNT, (void *)&counter, sizeof(counter))) {
			snprintf(rValueAdslDrv, ADSL_DRV_RETURN_LEN, "%d", counter.uas);
		}
		else
			ret = rGetFail;
	}
	*/
	else
		ret = rGetFail;
	
	return ret;
}
#endif

#if 0 // DSL driver page
char setAdslDrvInfo(char *name, char* value)
{
	int vInt;
	unsigned char vChar;

	printf("setAdslDrvInfo name(%s), value(%s)\n", name, value);
	
	if ( !strcmp(name, "adsl-drv-modem-start")) {
		sscanf(value, "%d", &vInt);
		if(adsl_drv_get(RLCM_PHY_START_MODEM, (void *)&vInt, 4)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-modem-enable")) {
		if(adsl_drv_get(RLCM_PHY_ENABLE_MODEM, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-modem-disable")) {
		if(adsl_drv_get(RLCM_PHY_DISABLE_MODEM, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-retrain")) {
		if(adsl_drv_get(RLCM_MODEM_RETRAIN, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-mode")) {
		sscanf(value, "%d", &vInt);
		vChar = (unsigned char)vInt;
		if ( mib_set( MIB_ADSL_MODE, (void *)&vChar))
			return 1;
//		if(adsl_drv_get(RLCM_SET_ADSL_MODE, (void *)&vInt, 4)) {
//			return 1;
//		}
	} 

	else if ( !strcmp(name, "adsl-drv-txpower")) {
		sscanf(value, "%d", &vInt);
		if(adsl_drv_get(RLCM_INC_TX_POWER, (void *)&vInt, 4)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-tuneper")) {
		sscanf(value, "%d", &vInt);
		if(adsl_drv_get(RLCM_TUNE_PERF, (void *)&vInt, 4)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-bitswap-enable")) {
		if(adsl_drv_get(RLCM_ENABLE_BIT_SWAP, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-bitswap-disable")) {
		if(adsl_drv_get(RLCM_DISABLE_BIT_SWAP, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-pilotrelocat-enable")) {
		if(adsl_drv_get(RLCM_ENABLE_PILOT_RELOCATION, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-pilotrelocat-disable")) {
		if(adsl_drv_get(RLCM_DISABLE_PILOT_RELOCATION, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-trellis-enable")) {
		if(adsl_drv_get(RLCM_ENABLE_TRELLIS, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-trellis-disable")) {
		if(adsl_drv_get(RLCM_DISABLE_TRELLIS, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-vendorid")) {
		sscanf(value, "%d", &vInt);
		if(adsl_drv_get(RLCM_SET_VENDOR_ID, (void *)&vInt, 4)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-debugmode")) {
		sscanf(value, "%d", &vInt);
		if(adsl_drv_get(RLCM_DEBUG_MODE, (void *)&vInt, 4)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-testpsd")) {
		sscanf(value, "%d", &vInt);
		if(adsl_drv_get(RLCM_TEST_PSD, (void *)&vInt, 4)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-log-enable")) {
		if(adsl_drv_get(RLCM_ENABLE_ADSL_LOG, NULL, 0)) {
			return 1;
		}
	} 

	else if ( !strcmp(name, "adsl-drv-log-disable")) {
		if(adsl_drv_get(RLCM_DISABLE_ADSL_LOG, NULL, 0)) {
			return 1;
		}
	} 
	
	return 0;
}

int getAdslDrvSnrTbl(short* tbl)
{
#ifdef EMBED
	if(!adsl_drv_get(RLCM_GET_CHANNEL_SNR, (void *)tbl, RLCM_GET_CHANNEL_SNR_SIZE)) {
		return 0;
	}
#else
#define ADSL_MAX_DATA_VALUE_1	512

	int	seed;
	int   len = RLCM_GET_CHANNEL_SNR_SIZE/sizeof(short);
	int  idx;
	for(idx=0;idx<len;idx++) {
		seed = (int) random();
		tbl[idx] = (short) (seed %= ADSL_MAX_DATA_VALUE_1);
	}
#endif

	return RLCM_GET_CHANNEL_SNR_SIZE/sizeof(short);
}

int getAdslDrvBitLoadTbl(short* tbl)
{
#ifdef EMBED
	if(!adsl_drv_get(RLCM_GET_CHANNEL_BITLOAD, (void *)tbl, RLCM_GET_CHANNEL_BITLOAD_SIZE)) {
		return 0;
	}
#else
#define ADSL_MAX_DATA_VALUE_2	12

	int	seed;
	int   len = RLCM_GET_CHANNEL_BITLOAD_SIZE/sizeof(short);
	int  idx;
	for(idx=0;idx<len;idx++) {
		seed = (int) random();
		tbl[idx] = (short) (seed %= ADSL_MAX_DATA_VALUE_2);
	}
#endif
	return RLCM_GET_CHANNEL_BITLOAD_SIZE/sizeof(short);
}
#endif

