
/*
 *      Web server handler routines for ACL stuffs
 *
 */


/*-- System inlcude files --*/
#include <net/if.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <net/route.h>

/*-- Local inlcude files --*/
#include "../webs.h"
#include "webform.h"
#include "mib.h"
#include "utility.h"
#include "../defs.h"

#ifdef MAC_ACL
int obtainMacAclEntry(webs_t wp, MIB_CE_ACL_MAC_Tp pEntry)
{
	char_t	*str;
	char macAddr[18];
	char tmpBuf[100];
	
	str = websGetVar(wp, T("aclMac"), T(""));
	if (str[0]) {
		int i;
		memcpy(macAddr, str, 17);
		for(i=0;i<17;i++){
			if((i+1)%3 != 0)
				macAddr[i-(i+1)/3] = macAddr[i];
		}
		macAddr[12] = '\0';
		if ( !string_to_hex(macAddr, pEntry->macAddr, 12)) {
			strcpy(tmpBuf, T(Tinvalid_source_mac));
			goto setErr_route;
		}
	}
	str = websGetVar(wp, T("enablemac"), T(""));
	if ( str[0] == '1' ) {
		pEntry->Enabled = 1;
	}else
		pEntry->Enabled = 0;
	
	str = websGetVar(wp, T("macinterface"), T(""));			
	if ( str[0]=='0' ) {
		pEntry->Interface = IF_DOMAIN_LAN;   // LAN
	} else if ( str[0]=='1') {
		pEntry->Interface = IF_DOMAIN_WAN;   // WAN
	} else {
		strcpy(tmpBuf, T(strSetInterfaceerror));
		goto setErr_route;		
	}
	return 1;

setErr_route:
	ERR_MSG(tmpBuf);
	return 0;
}
#endif

int obtainIpAclEntry(webs_t wp, MIB_CE_ACL_IP_Tp pEntry)
{
	char_t	*str;
	char tmpBuf[100];
	int isnet;
	struct sockaddr acl_ip;
	struct in_addr *addr;
	
	str = websGetVar(wp, T("aclIP"), T(""));	
	if (str[0]){
		if ((isnet = INET_resolve(str, &acl_ip)) < 0) {
			snprintf(tmpBuf, 100, "Error: can't resolve %s", str);
			goto setErr_route;
		}
	
		// add into configuration (chain record)
		addr = (struct in_addr *)&pEntry->ipAddr;
		*addr = ((struct sockaddr_in *)&acl_ip)->sin_addr;
	}

	str = websGetVar(wp, T("enable"), T(""));
	if ( str && str[0] ) {
		pEntry->Enabled = 1;
	}
	
	str = websGetVar(wp, T("interface"), T(""));			
	if ( str[0]=='0' ) {		
		pEntry->Interface = IF_DOMAIN_LAN;   // LAN
	} else if ( str[0]=='1') {		
		pEntry->Interface = IF_DOMAIN_WAN;   // WAN
	} else {
		strcpy(tmpBuf, T(strSetInterfaceerror));
		goto setErr_route;		
	}
	return 1;
	
setErr_route:
	ERR_MSG(tmpBuf);
	return 0;

}

#ifdef IP_ACL
///////////////////////////////////////////////////////////////////
 #if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
int aclapplyflag=0;
#endif
void formACL(webs_t wp, char_t *path, char_t *query)
{
	char_t	*str, *submitUrl;
	char tmpBuf[100];
	struct rtentry rt;
	int xflag, isnet;
	int skfd;
	struct sockaddr acl_ip;
	unsigned char aclcap;
	struct in_addr *addr;
	MIB_CE_ACL_IP_T entry;
#ifdef MAC_ACL
	MIB_CE_ACL_MAC_T macEntry;	
#endif
#ifndef NO_ACTION
	int pid;
#endif
	unsigned char vChar;

	// Set ACL Capability
	str = websGetVar(wp, T("apply"), T(""));
	if (str[0]) {
		str = websGetVar(wp, T("aclcap"), T(""));
		if (str[0]) {
			if (str[0] == '0')
				aclcap = 0;
			else
				aclcap = 1;
			if ( !mib_set(MIB_ACL_CAPABILITY, (void *)&aclcap)) {
				strcpy(tmpBuf, T(strSetACLCAPerror));
				goto setErr_route;
			}
		}
		goto setOk_route;
 	}
 #if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
	str = websGetVar(wp, T("effect"), T(""));
	if(str[0])
	{
		aclapplyflag = 0;
		goto setEffect_route;
	}
 #endif	
	// Delete IP
	str = websGetVar(wp, T("delIP"), T(""));
	if (str[0]) {
		unsigned int i;
		unsigned int idx;
		MIB_CE_ACL_IP_T Entry;
		unsigned int totalEntry = mib_chain_total(MIB_ACL_IP_TBL); /* get chain record size */
		struct in_addr dest;		
		char sdest[16];
		
		str = websGetVar(wp, T("select"), T(""));		
		
		if (str[0]) {
			for (i=0; i<totalEntry; i++) {
				idx = totalEntry-i-1;			
				snprintf(tmpBuf, 4, "s%d", idx);
				
				if ( !gstrcmp(str, T(tmpBuf)) ) {
					struct sockaddr_in *s_in;
					/* get the specified chain record */
					if (!mib_chain_get(MIB_ACL_IP_TBL, idx, (void *)&Entry)) {
						strcpy(tmpBuf, errGetEntry);
						goto setErr_route;
					}
					
					//printf("Entry.ipAddr = %s\n", Entry.ipAddr);
					dest.s_addr = *(unsigned long *)Entry.ipAddr;
		
					// inet_ntoa is not reentrant, we have to
					// copy the static memory before reuse it
					strcpy(sdest, inet_ntoa(dest));
					
					// iptables -D INPUT -s xxx.xxx.xxx.xxx		
					//va_cmd(IPTABLES, 6, 1, "-D", "INPUT", "-s", sdest, "-j", "ACCEPT");	
					
					// delete from chain record
					if(mib_chain_delete(MIB_ACL_IP_TBL, idx) != 1) {
						strcpy(tmpBuf, T(strDelChainerror));
						goto setErr_route;
					}					
				}
			} // end of for
		}		
		goto setOk_route;
	}

	//Delete MAC
#ifdef MAC_ACL
	str = websGetVar(wp, T("delMac"), T(""));
	if (str[0]) {
		unsigned int i;
		unsigned int idx;
		MIB_CE_ACL_MAC_T Entry;
		unsigned int totalEntry = mib_chain_total(MIB_ACL_MAC_TBL); /* get chain record size */
		
		str = websGetVar(wp, T("selectMac"), T(""));		
		
		if (str[0]) {
			for (i=0; i<totalEntry; i++) {
				idx = totalEntry-i-1;
				snprintf(tmpBuf, 4, "ms%d", idx);
				
				if ( !gstrcmp(str, T(tmpBuf)) ) {
					// delete from chain record
					if(mib_chain_delete(MIB_ACL_MAC_TBL, idx) != 1) {
						strcpy(tmpBuf, T(strDelChainerror));
						goto setErr_route;
					}
				}
			} // end of for
		}
		goto setOk_route;
	}
#endif	
	
#ifdef MAC_ACL
	// Update
	str = websGetVar(wp, T("updateMacACL"), T(""));
	if (str[0]) {
		unsigned int i, j;
		unsigned int idx;
		MIB_CE_ACL_MAC_T Entry, tmpEntry;
		unsigned int totalEntry = mib_chain_total(MIB_ACL_MAC_TBL); /* get chain record size */

		memset(&macEntry, 0, sizeof(MIB_CE_ACL_MAC_T));
		if ( !obtainMacAclEntry(wp, &macEntry))
			return;
		
		str = websGetVar(wp, T("selectMac"), T(""));
		if (str[0]) {
			for (i=0; i<totalEntry; i++) {
				idx = totalEntry-i-1;			
				snprintf(tmpBuf, 4, "ms%d", idx);
				
				if ( !gstrcmp(str, T(tmpBuf)) ) {
					//check if the mac existed.
					for (j=0; j<totalEntry; j++) {
						if (j==idx)
							continue;
						if(!mib_chain_get(MIB_ACL_MAC_TBL, j, (void*)&tmpEntry))
							continue;
						if ( (tmpEntry.macAddr[0]==macEntry.macAddr[0]) && 
							(tmpEntry.macAddr[1]==macEntry.macAddr[1]) &&
							(tmpEntry.macAddr[2]==macEntry.macAddr[2]) &&
							(tmpEntry.macAddr[2]==macEntry.macAddr[3]) &&
							(tmpEntry.macAddr[2]==macEntry.macAddr[4]) &&
							(tmpEntry.macAddr[2]==macEntry.macAddr[5])) {
							snprintf(tmpBuf, 100, strMacExist, str);
							goto setErr_route;
						}
					}
					
					mib_chain_update(MIB_ACL_MAC_TBL, &macEntry, idx);
					break;
				}
			} // end of for
		}
		
		goto setOk_route;
	}
	
	// Add
	str = websGetVar(wp, T("addMac"), T(""));
	if (str[0]) {
		int mibtotal,i;
		MIB_CE_ACL_MAC_T tmpentry;

		memset(&macEntry, 0, sizeof(MIB_CE_ACL_MAC_T));
		if ( !obtainMacAclEntry(wp, &macEntry))
			return;
		
		mibtotal = mib_chain_total(MIB_ACL_MAC_TBL);
		for(i=0;i<mibtotal;i++)
		{
			if(!mib_chain_get(MIB_ACL_MAC_TBL,i,(void*)&tmpentry))
				continue;
			if( (tmpentry.macAddr[0]==macEntry.macAddr[0]) &&
				(tmpentry.macAddr[1]==macEntry.macAddr[1]) &&
				(tmpentry.macAddr[2]==macEntry.macAddr[2]) &&
				(tmpentry.macAddr[3]==macEntry.macAddr[3]) &&
				(tmpentry.macAddr[4]==macEntry.macAddr[4]) &&
				(tmpentry.macAddr[5]==macEntry.macAddr[5]) )
			{
				snprintf(tmpBuf, 100, strMacExist, str);
				goto setErr_route;
			}
		}
			
		if(mib_chain_add(MIB_ACL_MAC_TBL, (unsigned char*)&macEntry) != 1){
			websWrite(wp, T("%s"), strAddChainerror);
			return;
		}
		
		goto setOk_route;
	}
#endif
	
	// Update
	str = websGetVar(wp, T("updateACL"), T(""));
	if (str[0]) {
		unsigned int i;
		unsigned int idx;
		MIB_CE_ACL_IP_T Entry;
		unsigned int totalEntry = mib_chain_total(MIB_ACL_IP_TBL); /* get chain record size */
		struct in_addr dest;		
		char sdest[16];

		memset(&entry, 0, sizeof(MIB_CE_ACL_IP_T));
		if ( !obtainIpAclEntry(wp, &entry))
			return;
		
		str = websGetVar(wp, T("select"), T(""));		
		
		if (str[0]) {
			for (i=0; i<totalEntry; i++) {
				idx = totalEntry-i-1;			
				snprintf(tmpBuf, 4, "s%d", idx);
				
				if ( !gstrcmp(str, T(tmpBuf)) ) {					
					mib_chain_update(MIB_ACL_IP_TBL, &entry, idx);
					//ql_xu add
					break;
				}
			} // end of for
		}
		
		goto setOk_route;
	}
	
	// Add
	str = websGetVar(wp, T("addIP"), T(""));
	if (str[0]) {
		int mibtotal,i;
		MIB_CE_ACL_IP_T tmpentry;

		memset(&entry, 0, sizeof(MIB_CE_ACL_IP_T));
		if ( !obtainIpAclEntry(wp, &entry))
			return;

		mibtotal = mib_chain_total(MIB_ACL_IP_TBL);
		for(i=0;i<mibtotal;i++)
		{
			if(!mib_chain_get(MIB_ACL_IP_TBL,i,(void*)&tmpentry))
				continue;
			if((*(int*)(entry.ipAddr))==(*(int*)(tmpentry.ipAddr)))
			{
				snprintf(tmpBuf, 100, strIpExist, str);
				goto setErr_route;
			}
		}
			
		if(mib_chain_add(MIB_ACL_IP_TBL, (unsigned char*)&entry) != 1){
			websWrite(wp, T("%s"), strAddChainerror);
			return;
		}		
	}


setOk_route:
	/* upgdate to flash */
//	mib_update(CURRENT_SETTING);
 #if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
	aclapplyflag = 1;
#endif

#ifndef NO_ACTION
	pid = fork();
        if (pid)
                waitpid(pid, NULL, 0);
        else if (pid == 0) {
		snprintf(tmpBuf, 100, "%s/%s", _CONFIG_SCRIPT_PATH, _CONFIG_SCRIPT_PROG);
#ifdef HOME_GATEWAY
		execl( tmpBuf, _CONFIG_SCRIPT_PROG, "gw", "bridge", NULL);
#else
		execl( tmpBuf, _CONFIG_SCRIPT_PROG, "ap", "bridge", NULL);
#endif
                exit(1);
        }
#endif

	submitUrl = websGetVar(wp, T("submit-url"), T(""));
	if (submitUrl[0])
		websRedirect(wp, submitUrl);
	else
		websDone(wp, 200);
  	return;
 #if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
setEffect_route:
	#ifdef __CLOSE_INTERFACE_BEFORE_WRITE_
			/*to avoid the dead when wrriting the flash*/
	itfcfg("sar", 0);
	itfcfg("eth0", 0);
#ifdef WLAN_SUPPORT
	itfcfg("wlan0", 0);
#endif
#endif /*__CLOSE_INTERFACE_BEFORE_WRITE_*/
	mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);

#ifdef __CLOSE_INTERFACE_BEFORE_WRITE_
	itfcfg("sar", 1);
	itfcfg("eth0", 1);
#ifdef WLAN_SUPPORT
	vChar=0;
	mib_get( MIB_WLAN_DISABLED, (void *)&vChar);
	if( vChar==0 )	itfcfg("wlan0", 1);
#endif //WLAN_SUPPORT
#endif /*__CLOSE_INTERFACE_BEFORE_WRITE_*/
//		RECONNECT_MSG(strIp);
//		req_flush(wp);
	restart_acl();
	submitUrl = websGetVar(wp, T("submit-url"), T(""));
	OK_MSG(submitUrl);
	return;
#endif
setErr_route:
	ERR_MSG(tmpBuf);
}

int showACLTable(int eid, webs_t wp, int argc, char_t **argv)
{
	int nBytesSent=0;
	
	unsigned int entryNum, i;
	MIB_CE_ACL_IP_T Entry;
	unsigned long int d,g,m;
	struct in_addr dest;
	struct in_addr gw;
	struct in_addr mask;
	char sdest[16], sgw[16];
	
	entryNum = mib_chain_total(MIB_ACL_IP_TBL);
 #if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
      unsigned char validAclListNum=0;
#endif	
	nBytesSent += websWrite(wp, T("<tr><font size=1>"
	"<td align=center width=\"5%%\" bgcolor=\"#808080\">Select</td>\n"
	"<td align=center width=\"20%%\" bgcolor=\"#808080\">state</td>\n"
	"<td align=center width=\"20%%\" bgcolor=\"#808080\">Interface</td>\n"
	"<td align=center width=\"20%%\" bgcolor=\"#808080\">IP Address</td></font></tr>\n"));
	for (i=0; i<entryNum; i++) {

		if (!mib_chain_get(MIB_ACL_IP_TBL, i, (void *)&Entry))
		{
  			websError(wp, 400, T("Get chain record error!\n"));
			return;
		}
			
		dest.s_addr = *(unsigned long *)Entry.ipAddr;
		
		// inet_ntoa is not reentrant, we have to
		// copy the static memory before reuse it
		strcpy(sdest, inet_ntoa(dest));
		
	
		nBytesSent += websWrite(wp, T("<tr>"
			"<td align=center width=\"5%%\" bgcolor=\"#C0C0C0\"><input type=\"radio\" name=\"select\""
			" value=\"s%d\""
			" onClick=\"postACL(%d, %d, '%s')\""
			"></td>\n"
#if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
			"<input type=\"hidden\" id=\"aclItemEnable%d\" value=%d>\n"
#endif
			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\" ><font size=\"2\"><b>%s</b></font></td>\n"
			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\"><b>%s</b></font></td>\n"
			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\"><b>%s</b></font></td>"
			"</tr>\n"),
			i, 
			Entry.Enabled, (Entry.Interface == IF_DOMAIN_LAN)?0:1, sdest,
#if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
			i, Entry.Enabled,Entry.Enabled ? TAdmin_acl_enable : TAdmin_acl_disable, (Entry.Interface == IF_DOMAIN_LAN)? "LAN" : "WAN", sdest);
#else		
			Entry.Enabled ? TAdmin_acl_enable: TAdmin_acl_disable, (Entry.Interface == IF_DOMAIN_LAN)? "LAN" : "WAN", sdest);
#endif
#if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
		if(Entry.Enabled) validAclListNum++;
#endif
	}
#if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
	nBytesSent += websWrite(wp, T("<input type=\"hidden\" value=%d name=\"listnumber\">\n"), entryNum);
	 nBytesSent += websWrite(wp, T("<input type=\"hidden\" value=%d name=\"validAclListNum\">\n"), validAclListNum);
#endif
	
	return 0;
}
#endif

#ifdef MAC_ACL
int showACLMacTable(int eid, webs_t wp, int argc, char_t * * argv)
{
	int nBytesSent=0;
	unsigned int entryNum, i;
	MIB_CE_ACL_MAC_T Entry;
	char tmpBuf[100];
 #if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
	unsigned char validMacAclListNum=0;
#endif	
	
	entryNum = mib_chain_total(MIB_ACL_MAC_TBL);
	nBytesSent += websWrite(wp, T("<tr><font size=1>"
		"<td align=center width=\"5%%\" bgcolor=\"#808080\">Select</td>\n"
		"<td align=center width=\"20%%\" bgcolor=\"#808080\">state</td>\n"
		"<td align=center width=\"20%%\" bgcolor=\"#808080\">Interface</td>\n"
		"<td align=center width=\"20%%\" bgcolor=\"#808080\">MAC Address</td></font></tr>\n"));
	for (i=0; i<entryNum; i++) {

		if (!mib_chain_get(MIB_ACL_MAC_TBL, i, (void *)&Entry))
		{
  			websError(wp, 400, T(Tget_chain_err));
			return;
		}

		snprintf(tmpBuf, 100, T("%02x-%02x-%02x-%02x-%02x-%02x"),
			Entry.macAddr[0], Entry.macAddr[1], Entry.macAddr[2],
			Entry.macAddr[3], Entry.macAddr[4], Entry.macAddr[5]);
		
		nBytesSent += websWrite(wp, T("<tr>"
			"<td align=center width=\"5%%\" bgcolor=\"#C0C0C0\"><input type=\"radio\" name=\"selectMac\""
			" value=\"ms%d\""
			" onClick=\"postACL2(%d, %d, '%s')\""
			"></td>\n"
#if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
			"<input type=\"hidden\" id=\"aclMacItemEnable%d\" value=%d>\n"
#endif
			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\" ><font size=\"2\"><b>%s</b></font></td>\n"
			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\"><b>%s</b></font></td>\n"
			"<td align=center width=\"20%%\" bgcolor=\"#C0C0C0\"><font size=\"2\"><b>%s</b></font></td>"
			"</tr>\n"),
			i, 
			Entry.Enabled, (Entry.Interface == IF_DOMAIN_LAN)?0:1, tmpBuf,
#if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
			i, Entry.Enabled,
			Entry.Enabled?TAdmin_acl_enable: TAdmin_acl_disable , (Entry.Interface==IF_DOMAIN_LAN)?"LAN":"WAN", tmpBuf);
#else		
			Entry.Enabled?TAdmin_acl_enable: TAdmin_acl_disable , (Entry.Interface==IF_DOMAIN_LAN)?"LAN":"WAN", tmpBuf);
#endif
#if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
		if(Entry.Enabled) validMacAclListNum++;
#endif
	}
#if defined( ZTE_531B_BRIDGE_SC)||defined(ZTE_GENERAL_ROUTER_SC)
	nBytesSent += websWrite(wp, T("<input type=\"hidden\" value=%d name=\"Maclistnumber\">\n"), entryNum);
	nBytesSent += websWrite(wp, T("<input type=\"hidden\" value=%d name=\"validMacAclListNum\">\n"), validMacAclListNum);
#endif
	
	return 0;
}

#endif


