/*
 *  Boa, an http server
 *  Copyright (C) 1995 Paul Phillips <psp@well.com>
 *  Authorization "module" (c) 1998,99 Martin Hinner <martin@tdp.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

extern char *mgmtUserName(void);
extern char *mgmtPassword(void);

#include <stdio.h>
#include <fcntl.h>
#ifdef __UC_LIBC__
#include <unistd.h>
#else
#include <crypt.h>
#endif
#include "syslog.h"
#include "md5.h"
#include "boa.h"
#ifdef SHADOW_AUTH
#include <shadow.h>
#endif
#ifdef OLD_CONFIG_PASSWORDS
#include <crypt_old.h>
#endif
#ifdef EMBED
#include <sys/types.h>
#include <pwd.h>
#include <config/autoconf.h>
#endif

#ifdef SECURITY_COUNTS
#include "../../login/logcnt.c"
#endif

#ifdef EMBED
// Added by Mason Yu for 2 level web page
#include "./LINUX/mib.h"

// Added by Mason Yu
//extern char usName[MAX_NAME_LEN];
#endif

//add by xl_yue
#ifdef ACCOUNT_LOGIN_CONTROL
/*
extern char suName[MAX_NAME_LEN];
extern struct account_info su_account;
extern struct account_info us_account;
extern time_t time_counter;
extern struct errlogin_entry * errlogin_list;
*/
#define MAX_LOG_NUM 5
#endif

#ifdef USE_AUTH

// Added by Mason Yu
#ifdef BOA_UNAUTH_FILE
#ifndef CONFIG_GUI_WEB
#define MAX_UNAUTH_FILE 22
#else
#define MAX_UNAUTH_FILE 40
#endif
char *UNAUTHFILE[MAX_UNAUTH_FILE] = {
	"/",
	"/admin/title.html",
	"/admin/status.asp",
	"/admin/share.js",
	"/share.js",
	"/status.asp",
	"/goform/admin/formStatus",
	"/admin/redirect.asp",
#ifndef CONFIG_GUI_WEB
	"/code.asp",
	"/admin/code_user.asp",
	"/admin/graphics/topbar.gif",
	"/menu-images/menu_root.gif",
	"/menu-images/menu_tee.gif",
	"/menu-images/menu_link_default.gif",
	"/menu-images/menu_folder_closed.gif",
	"/menu-images/menu_bar.gif",
	"/menu-images/menu_corner.gif",
	"/admin/menu-images/menu_root.gif",
	"/admin/menu-images/menu_tee.gif",
	"/admin/menu-images/menu_link_default.gif",
	"/admin/menu-images/menu_folder_closed.gif",
	"/admin/menu-images/menu_bar.gif",
	"/admin/menu-images/menu_corner.gif",
#else
	"/admin/title.asp",
	"/blank.html",
	"/admin/css.css",
	"/admin/stm31.js",
	"/qsetup.asp",
	"/qlogin.asp",
	"/popup.asp",
	"/notconnection.asp",
	"/style.css",
	"/images/spacer.gif",
	"/images/bg.gif",
	"/admin/images/bg1.gif",
	"/admin/images/bg3.gif",
	"/admin/images/logo.gif",
	"/admin/images/spacer.gif",
	"/admin/images/bg4.gif",
	"/admin/images/img-status-b.gif",
	"/admin/images/img-status-a.gif",
	"/admin/images/img-setup-b.gif",
	"/admin/images/img-setup-a.gif",
	"/admin/images/img-lan-b.gif",
	"/admin/images/img-lan-a.gif",
	"/admin/images/img-internet-b.gif",
	"/admin/images/img-internet-a.gif",
	"/admin/images/img-firewall-b.gif",
	"/admin/images/img-firewall-a.gif",
	"/admin/images/img-advance-b.gif",
	"/admin/images/img-advance-a.gif",
	"/admin/images/img-admin-b.gif",
	"/admin/images/img-admin-a.gif",
	"/admin/images/img-wireless-b.gif",
	"/admin/images/img-wireless-a.gif",
	"/goform/formPPPAuth"
#endif
};
#endif

struct _auth_dir_ {
	char *directory;
	FILE *authfile;
	int dir_len;
	struct _auth_dir_ *next;
};

typedef struct _auth_dir_ auth_dir;

static auth_dir *auth_hashtable [AUTH_HASHTABLE_SIZE];
#ifdef OLD_CONFIG_PASSWORDS
char auth_old_password[16];
#endif

/*
 * Name: get_auth_hash_value
 *
 * Description: adds the ASCII values of the file letters
 * and mods by the hashtable size to get the hash value
 */
inline int get_auth_hash_value(char *dir)
{
#ifdef EMBED
	return 0;
#else
	unsigned int hash = 0;
	unsigned int index = 0;
	unsigned char c;

	hash = dir[index++];
	while ((c = dir[index++]) && c != '/')
		hash += (unsigned int) c;

	return hash % AUTH_HASHTABLE_SIZE;
#endif
}

/*
 * Name: auth_add
 *
 * Description: adds 
 */
void auth_add(char *directory,char *file)
{
	auth_dir *new_a,*old;
	
	old = auth_hashtable [get_auth_hash_value(directory)];
	while (old)
	{
		if (!strcmp(directory,old->directory))
			return;
		old = old->next;
	}	
	
	new_a = (auth_dir *)malloc(sizeof(auth_dir));
	/* success of this call will be checked later... */
	new_a->authfile = fopen(file,"rt");
	new_a->directory = strdup(directory);
	new_a->dir_len = strlen(directory);
	new_a->next = auth_hashtable [get_auth_hash_value(directory)];
	auth_hashtable [get_auth_hash_value(directory)] = new_a;
}

void auth_check()
{
	int hash;
	auth_dir *cur;
	
	for (hash=0;hash<AUTH_HASHTABLE_SIZE;hash++)
	{
  	cur = auth_hashtable [hash];
	  while (cur)
		{
			if (!cur->authfile)
			{
				log_error_time();
				fprintf(stderr,"Authentication password file for %s not found!\n",
						cur->directory);
			}
			cur = cur->next;
		}
	}
}

#ifdef LDAP_HACK
#include <lber.h>
#include <ldap.h>
#include <sg_configdd.h>
#include <sg_confighelper.h>
#include <sg_users.h>

/* Return a positive, negative or not possible result to the LDAP auth for
 * the specified user.
 */
static int ldap_auth(const char *const user, const char *const pswd) {
	static time_t last;
	static char *prev_user;
	LDAP *ld;
	int ldap_ver, r;
	char f[256];
	ConfigHandleType *c = config_load(amazon_ldap_config_dd);

	/* Don't repeat query too often if the user name hasn't changed */
	if (last && prev_user &&
			time(NULL) < (last + config_get_int(c, AMAZON_LDAP_CACHET)) &&
			strcmp(prev_user, user) == 0) {
		config_free(c);
		last = time(NULL);
		return 1;
	}
	if (prev_user != NULL)   { free(prev_user);	prev_user = NULL;   }
	last = 0;

	if ((ld = ldap_init(config_get_string(c, AMAZON_LDAP_HOST),
			config_get_int(c, AMAZON_LDAP_PORT))) == NULL) {
		syslog(LOG_ERR, "unable to initialise LDAP");
		config_free(c);
		return 0;
	}
	if (ldap_set_option(ld, LDAP_OPT_REFERRALS, LDAP_OPT_OFF) != LDAP_SUCCESS) {
		syslog(LOG_ERR, "unable to set LDAP referrals off");
		config_free(c);
		ldap_unbind(ld);
		return 0;
	}
	ldap_ver = config_get_int(c, AMAZON_LDAP_VERSION);
	if (ldap_ver > 0 && ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &ldap_ver) != LDAP_SUCCESS) {
		syslog(LOG_ERR, "unable to set LDAP version %d", ldap_ver);
		config_free(c);
		ldap_unbind(ld);
		return 0;
	}
	snprintf(f, sizeof f, config_get_string(c, AMAZON_LDAP_BIND_DN), user);
	r = ldap_simple_bind_s(ld, f, pswd);
	if (r != LDAP_SUCCESS) {
		syslog(LOG_ERR, "unable to connect to LDAP (%s)", ldap_err2string(r));
		config_free(c);
		ldap_unbind(ld);
		return (r==LDAP_INVALID_CREDENTIALS)?-1:0;
	}
	config_free(c);
	ldap_unbind(ld);
	/* Caching timing details for next time through */
	prev_user = strdup(user);
	last = time(NULL);
	return 1;
}
static unsigned char ldap_succ;
#endif



/*
 * Name: auth_check_userpass
 *
 * Description: Checks user's password. Returns 0 when sucessful and password
 * 	is ok, else returns nonzero; As one-way function is used RSA's MD5 w/
 *  BASE64 encoding.
#ifdef EMBED
 * On embedded environments we use crypt(), instead of MD5.
#endif
 */
int auth_check_userpass(char *user,char *pass,FILE *authfile, request * req)
{		
	
#ifdef LDAP_HACK	
	
	/* Yeah, code before declarations will fail on older compilers... */
	switch (ldap_auth(user, pass)) {
	case -1:	ldap_succ = 0;				return 1;
	case 0:		ldap_succ = strcmp(user, "root")?0:1;	break;
	case 1:
		ldap_succ = 1;
		if (start_user_update(0) == 0)
			done_user_update(set_user_password(user, pass, 0)==0?1:0);
		return 0;
	}
#endif
#ifdef SHADOW_AUTH
	struct spwd *sp;	
	
	sp = getspnam(user);
	if (!sp)
		return 2;

	if (!strcmp(crypt(pass, sp->sp_pwdp), sp->sp_pwdp))
		return 0;

#else

//#ifndef EMBED
#if 1

#ifdef EMBED
	char temps[0x100],*pwd;
	struct MD5Context mc;
 	unsigned char final[16];
	char encoded_passwd[0x40];
	// Added by Mason Yu (2 leval)
	FILE *fp;
	char disabled;
	
   	/* Encode password ('pass') using one-way function and then use base64
	encoding. */		 
	
	// Added by Mason Yu(2 level)	
	//if ( strcmp("user", user)==0 && strcmp(req->request_uri, "/")==0 ) {	
		
	// Added by Mason Yu for boa memory leak
	// User access web every time, the boa will auth once. And use strdup to allocate memory for directory_index.
	// So We should free the memory space of older directory_index to avoid memory leak.
	if (directory_index) free(directory_index);
		
	if ( strcmp(usName, user)==0 ) {
		//printf("req->request_uri=%s\n", req->request_uri);
		directory_index = strdup("/admin/index_user.html");
		if ( strcmp(req->request_uri, "/")==0 ) {		
#ifdef WLAN_SUPPORT
		        // Get WLAN Interface status					
		        if ( mib_get(MIB_WLAN_DISABLED, (void *)&disabled) == 0) {
  		        	printf("Get WLAN interface flag error!\n");
  		        	return 1;			
		        }
		        
		        if ( disabled == 1 ) {
		        	printf("WLAN Interface is disabled !\n");
		        	return 1;
		        }else {
#endif	        					
				// Added by Mason Yu for web page via serverhost						
		        	//directory_index = strdup("/admin/index_user.html");	        	
		        	
		        	fp=fopen("/var/boaUser.passwd", "r");		        	
	        		authfile = fp;  	        			        			
#ifdef WLAN_SUPPORT
		        }
#endif

		}
		
	}else if (strcmp("adsl", user) == 0){
		char Passwd[50];
		
		if ( mib_get(MIB_SUPER_PASSWORD, (void *)Passwd) == 0) {
  			printf("Get Engineer Password error!\n");
  			return 1;			
		}
		
		if (strcmp(Passwd, pass) == 0) {
			directory_index = strdup("index.html");
			return 0;
		}		
	}else {		
		directory_index = strdup("index.html");	
	}	
	
		 
	MD5Init(&mc);
	{
	//char *pass="admin";
	MD5Update(&mc, pass, strlen(pass));
	}
	MD5Final(final, &mc);
	strcpy(encoded_passwd,"$1$");
	base64encode(final, encoded_passwd+3, 16);

	DBG(printf("auth_check_userpass(%s,%s,...);\n",user,pass);)	
	
	fseek(authfile,0,SEEK_SET);	
	while (fgets(temps,0x100,authfile))	
	{
		if (temps[strlen(temps)-1]=='\n')
			temps[strlen(temps)-1] = 0;
		pwd = strchr(temps,':');
		if (pwd)
		{
			*pwd++=0;			
			if (!strcmp(temps,user))
			{
				if (!strcmp(pwd,encoded_passwd)) {
					// Added by Mason Yu for web page via serverhost
					if (!strcmp(usName, user) && strcmp(req->request_uri, "/")==0 )					
						fclose(fp);
					return 0;
				}	
			} else {
				// Modified by Mason Yu for multi user with passwd file.				
				continue;
				//return 2;
			}
		}
	}
	
	// Added by Mason Yu for web page via serverhost
	if (!strcmp(usName, user) && strcmp(req->request_uri, "/")==0)
		fclose(fp);
#else
//printf("user=%s, pass=%s name=%s pass2=%s\n",user,pass,mgmtUserName(),mgmtPassword());
	//if((strcmp(mgmtUserName(),user)==0)&&(strcmp(mgmtPassword(),pass)==0)) return 0;
	//else return 2;
	return 0;
#endif

#else
	struct passwd *pwp;

	pwp = getpwnam(user);
	if (pwp != NULL) {
		if (strcmp(crypt(pass, pwp->pw_passwd), pwp->pw_passwd) == 0)
			return 0;
	} else 
#ifdef OLD_CONFIG_PASSWORDS
	/* For backwards compatibility we allow the global root password to work too */
	if ((auth_old_password[0] != '\0') && 
			((*user == '\0') || (strcmp(user, "root") == 0))) {
		if (strcmp(crypt_old(pass,auth_old_password),auth_old_password) == 0 ||
				strcmp(crypt(pass,auth_old_password),auth_old_password) == 0) {
			strcpy(user, "root");
			return 0;
		}
	} else
#endif	/* OLD_CONFIG_PASSWORDS */
		return 2;

#endif	/* ! EMBED */
#endif	/* SHADOW_AUTH */
	return 1;
}

#ifdef ACCOUNT_LOGIN_CONTROL
//added by xl_yue
struct errlogin_entry * scan_errlog_list(request * req)
{
	struct errlogin_entry * perrlog;
	
	for(perrlog = errlogin_list;perrlog;perrlog = perrlog->next){
		if(!strcmp(perrlog->remote_ip_addr, req->remote_ip_addr))
			break;
	}

	return perrlog;
}
//added by xl_yue
int check_errlogin_list(request * req)
{
	struct errlogin_entry * perrlog;
	struct errlogin_entry * berrlog = NULL;

	for(perrlog = errlogin_list;perrlog;berrlog = perrlog,perrlog = perrlog->next){
		if(!strcmp(perrlog->remote_ip_addr, req->remote_ip_addr))
			break;
	}

	if(!perrlog)
		return 0;
	
	if(perrlog->login_count >= MAX_LOG_NUM){
		return -1;
	}
	else{//unlist and free
		if(perrlog == errlogin_list)
			errlogin_list = perrlog->next;
		else
			berrlog->next = perrlog->next;
		
		free(perrlog);
	}
	return 0;
}
#endif

int auth_authorize(request * req)
{
	int i;
	auth_dir *current;
  	int hash;
	char *pwd;
#ifndef ACCOUNT_LOGIN_CONTROL
	static char current_client[20];
#endif
	char auth_userpass[0x80];
	//added by xl_yue
#ifdef ACCOUNT_LOGIN_CONTROL
	struct account_info * current_account_info = NULL;
	struct errlogin_entry * err_login;
	int do_succeed_log = 0;
#endif

//xl_yue
#ifdef USE_LOGINWEB_OF_SERVER
	struct user_info * pUser_info;
	struct user_info * bUser_info = NULL;
#ifdef LOGIN_ERR_TIMES_LIMITED
	struct errlogin_entry * pErrlog_entry = NULL;
#endif
#endif

  DBG(printf("auth_authorize\n");)

	// Mason Yu
	//printf("auth_authorize(1): req->request_uri=%s\n", req->request_uri);
#ifdef BOA_UNAUTH_FILE
	for (i=0; i<MAX_UNAUTH_FILE; i++) {
		if ( strcmp(req->request_uri, UNAUTHFILE[i]) == 0 ) {
#ifndef CONFIG_GUI_WEB
			directory_index = strdup("/admin/index_user.html");
#endif
			//printf("This file need not auth!!\n");
			return 1;
		}
	}
#endif

	hash = get_auth_hash_value(req->request_uri);
  current = auth_hashtable[hash];
					
  while (current) {
		if (!memcmp(req->request_uri, current->directory,
								current->dir_len)) {
			if (current->directory[current->dir_len - 1] != '/' &&
				        req->request_uri[current->dir_len] != '/' &&
								req->request_uri[current->dir_len] != '\0') {				
				break;
			}
//xl_yue add
#ifndef USE_LOGINWEB_OF_SERVER

			if (req->authorization)
			{
				int denied = 1;
								
				if (current->authfile==0)
				{					
					send_r_error(req);
					return 0;
				}

				if (strncasecmp(req->authorization,"Basic ",6))
				{
					syslog(LOG_ERR, "Can only handle Basic auth\n");
					send_r_bad_request(req);					
					return 0;
				}
				
				base64decode(auth_userpass,req->authorization+6,sizeof(auth_userpass));
				
				if ( (pwd = strchr(auth_userpass,':')) == 0 )
				{
					syslog(LOG_ERR, "No user:pass in Basic auth\n");
					send_r_bad_request(req);					
					return 0;
				}
				
				*pwd++=0;

				// Modified by Mason Yu 
				//denied = auth_check_userpass(auth_userpass,pwd,current->authfile);					
				denied = auth_check_userpass(auth_userpass,pwd,current->authfile, req);
				
				
#ifdef SECURITY_COUNTS
				if (strncmp(get_mime_type(req->request_uri),"image/",6))
					access__attempted(denied, auth_userpass);
#endif

#ifdef ACCOUNT_LOGIN_CONTROL
				//added by xl_yue
				if(!denied){
					//check if this ip in errlogin list
					if(check_errlogin_list(req)){
						denied = 5;
						goto pass_check;
					}

					//for user
					if (strcmp(usName, auth_userpass)==0){
						current_account_info = &us_account;
//						printf("select us_account\n");
					}
					//for super user
					else if(strcmp(suName, auth_userpass)==0){
						current_account_info = &su_account;
//						printf("select su_account\n");
					}
					else{
						current_account_info = NULL;
//						printf("select none account\n");
					}
					
					if(current_account_info){
						if (strcmp(current_account_info->remote_ip_addr, req->remote_ip_addr) == 0){
//							printf("the same ip\n");
							if(current_account_info->account_timeout){			//need login again for no acction for 5 minutes 
								current_account_info->account_timeout = 0;
								current_account_info->last_time = time_counter;
								denied = 3;
//								printf("account timeout\n");
							}else{
//								printf("account not timeout\n");
								current_account_info->last_time = time_counter;
								current_account_info->account_busy = 1;
								//req->paccount_info = current_account_info;
								//current_account_info->refcnt++;
							}
						}else{
//							printf("not same ip\n");
							//only one user can login using the same account at the same time
							//if (us_account.account_busy || su_account.account_busy) {
							if(current_account_info->account_busy){
//								printf("account busy\n");
								denied = 4;
							}else{
//								printf("account not busy\n");
								current_account_info->account_timeout = 0;
								current_account_info->last_time = time_counter;
								current_account_info->account_busy = 1;
								do_succeed_log = 1;
								strncpy(current_account_info->remote_ip_addr, req->remote_ip_addr, sizeof(current_account_info->remote_ip_addr));
								//req->paccount_info = current_account_info;
								//current_account_info->refcnt++;
							}
						}
					}
				}else{
					/*login error three times,should wait 1 minutes */
					if((err_login = scan_errlog_list(req)) == NULL){
						err_login = (struct errlogin_entry*) malloc(sizeof(struct errlogin_entry));
						if (!err_login)
							die(OUT_OF_MEMORY);
						err_login->login_count = 0;
						strncpy(err_login->remote_ip_addr, req->remote_ip_addr, sizeof(err_login->remote_ip_addr));
						//list err_login to errlogin_list
						err_login->next = errlogin_list;
						errlogin_list = err_login;
					}
					if((++(err_login->login_count)) >= MAX_LOG_NUM){
						//if have login,then exit
						if(!strcmp(su_account.remote_ip_addr, req->remote_ip_addr)){
							su_account.account_timeout = 0;
							su_account.account_busy = 0;
							su_account.remote_ip_addr[0] = '\0';
						}
						if(!strcmp(us_account.remote_ip_addr, req->remote_ip_addr)){
							us_account.account_timeout = 0;
							us_account.account_busy = 0;
							us_account.remote_ip_addr[0] = '\0';
						}

						if(err_login->login_count == MAX_LOG_NUM)
							err_login->last_time = time_counter;

						denied = 5;
					}else{
						err_login->last_time = time_counter;
					}
				}
#endif

pass_check:
				if (denied) {
					switch (denied) {
						case 1:
							syslog(LOG_ERR, "Authentication attempt failed for %s from %s because: Bad Password\n",
									auth_userpass, req->remote_ip_addr);
							break;
						case 2:
							syslog(LOG_ERR, "Authentication attempt failed for %s from %s because: Invalid Username\n",
									auth_userpass, req->remote_ip_addr);
							break;
#ifdef ACCOUNT_LOGIN_CONTROL
						//added by xl_yue
						case 3:
							syslog(LOG_ERR, "Authentication attempt failed for %s from %s because:login has timeouted \n",auth_userpass, req->remote_ip_addr);
							break;
						//added by xl_yue
						case 4:
							syslog(LOG_ERR, "Authentication attempt failed for %s from %s because: another has logined\n",auth_userpass, req->remote_ip_addr);
							send_r_forbidden(req);
							return 0;
							break;
						//added by xl_yue
						case 5:
							syslog(LOG_ERR, "Authentication attempt failed for %s from %s because: have logined error for three times\n",auth_userpass, req->remote_ip_addr);
							send_r_forbidden(req);
							return 0;
							break;
#endif	
						}
#ifndef ACCOUNT_LOGIN_CONTROL
					bzero(current_client, sizeof(current_client));
#endif
					send_r_unauthorized(req,server_name);
					return 0;
				}
#ifdef ACCOUNT_LOGIN_CONTROL
				if(do_succeed_log){
					syslog(LOG_INFO, "Authentication successful for %s from %s\n", auth_userpass, req->remote_ip_addr);
				}
#else
				if (strcmp(current_client, req->remote_ip_addr) != 0) {
					strncpy(current_client, req->remote_ip_addr, sizeof(current_client));
					syslog(LOG_INFO, "Authentication successful for %s from %s\n", auth_userpass, req->remote_ip_addr);
				}
#endif
				/* Copy user's name to request structure */
#ifdef LDAP_HACK
				if (!ldap_succ) {
					strcpy(req->user, "noldap");
					syslog(LOG_INFO, "Access granted as noldap");
				} else
#endif
				strncpy(req->user, auth_userpass, 15);
				req->user[15] = '\0';
				return 1;
			}else
			{
				/* No credentials were supplied. Tell them that some are required */
#ifdef ACCOUNT_LOGIN_CONTROL				
				//added by xl_yue:if have login error for three times, forbid logining until 1 minute later
				if((err_login = scan_errlog_list(req)) != NULL){
					if(err_login->login_count >= MAX_LOG_NUM){
						send_r_forbidden(req);
						return 0;
					}
				}
#endif
				send_r_unauthorized(req,server_name);
				return 0;
			}
#else
//xl_yue
				pUser_info = search_login_list2(req, &bUser_info);
				if(pUser_info && pUser_info->timeout == 0){
					//user account can not access admin account directory
					if((strcmp(req->request_uri,"/")) && (!strcmp(current->directory,"/")) &&
						(!strcmp(pUser_info->directory,"/admin/index_user.html"))&& (!strcmp(pUser_info->directory,"/index.html"))){
						send_r_forbidden(req);
						return 0;
					}
					directory_index = pUser_info->directory;
					pUser_info->last_time = time_counter;
					return 1;
				}else{
#ifdef LOGIN_ERR_TIMES_LIMITED
					pErrlog_entry = search_errlog_list(req);
					if(pErrlog_entry){
						if(pErrlog_entry->login_count >= MAX_LOGIN_NUM){
							send_r_forbidden(req);
							return 0;
						}
					}
#endif
					if(!strcmp(req->request_uri,"/admin/login.asp")
						|| !strcmp(req->request_uri,"/admin/autologin.asp") 
						|| !strcmp(req->request_uri,"/goform/admin/formLogin") 
						|| !strcmp(req->request_uri,"/image/logo.gif") 
#ifdef EHOMECLIENT_SUPPORT
						|| !strcmp(req->request_uri,"/admin/e8clientlogin.asp")
						|| !strcmp(req->request_uri,"/goform/admin/e8client") 
#endif
						)
						return 1;
					
					directory_index = NULL;

					//if(strcmp(req->request_uri,"/")){
/*						
						req->response_status = R_FORBIDDEN;
						req_write(req, "<HTML><HEAD><TITLE>Forbidden</TITLE></HEAD>\n <BODY><H1>Forbidden </H1>\n");
						req_write(req, " Your requested URL is forbidden to access, maybe because you have not logined.\n");
 						req_write(req, " Please close this window and reopen your web browser to login.\n</BODY></HTML>\n");
						req_flush(req);
*/					
						req->buffer_end=0;
						req->response_status = R_MOVED_PERM;
						#if 0
						req_write(req, "<HTML><HEAD><TITLE>Login</TITLE></HEAD>\n"
									 "<BODY><BLOCKQUOTE>\n <h2><font color=\"#0000FF\">Login</font></h2>"
									 "You have not logined,please\n");
						req_write(req, "<A HREF=\"/admin/login.asp\" target=_top>login</A>.\n</BLOCKQUOTE></BODY></HTML>\n");
						#endif
						//lanzhd, e8b
						if(pUser_info)
						{
						#ifdef EHOMECLIENT_SUPPORT
							if(strstr(req->request_uri,"ehomeclient"))
								req_write(req, "<html><head><script language=\"javascript\">alert(\"¼ʱµ¼!\");top.location.href=\"/admin/e8clientlogin.asp\";</script></head><body></body></html>");
							else
						#endif
								req_write(req, "<html><head><script language=\"javascript\">alert(\"¼ʱµ¼!\");top.location.href=\"/admin/login.asp\";</script></head><body></body></html>");
							ulist_free_login_entry(bUser_info, pUser_info);
						}
						else
						{
						#ifdef EHOMECLIENT_SUPPORT
							if(strstr(req->request_uri,"ehomeclient"))
								req_write(req, "<html><head><script language=\"javascript\">top.location.href=\"/admin/e8clientlogin.asp\";</script></head><body></body></html>");
							else
						#endif
								req_write(req, "<html><head><script language=\"javascript\">top.location.href=\"/admin/login.asp\";</script></head><body></body></html>");
						}
						req_flush(req);
						return 0;
					//}
					
					//websRedirect(req, "/admin/login.asp");
					//return 0;
				}
#endif
		}
	  current = current->next;
  }
						
	return 1;
}

void dump_auth(void)
{
	int i;
	auth_dir *temp;

	for (i = 0; i < AUTH_HASHTABLE_SIZE; ++i) {
		if (auth_hashtable[i]) {
			temp = auth_hashtable[i];
			while (temp) {
				auth_dir *temp_next;

				if (temp->directory)
					free(temp->directory);
				if (temp->authfile)
					fclose(temp->authfile);
				temp_next = temp->next;
				free(temp);
				temp = temp_next;
			}
			auth_hashtable[i] = NULL;
		}
	}
}

#endif
